breeds [ moths darkness predator ]

moths-own [
    red-pigment-gene-frequency 
    blue-pigment-gene-frequency 
    green-pigment-gene-frequency
  ]

globals [
    total-caught 
    caught-in-batch 
    interval 
    ticks 
    last-ticks 
    batch-number 
    batch-size
    last-batch-caught 
    your-energy max-energy
    last-mouse-xcor 
    last-mouse-ycor
    night?
  ]

to setup
   ca
   change-environment
   make-initial-moths
   set ticks 0
   set batch-size 10
   set batch-number 0
   set interval 0
   set caught-in-batch 0
   set last-ticks 0
   set your-energy 100
   set max-energy 100
   set last-mouse-xcor mouse-xcor
   set last-mouse-ycor mouse-ycor
   set night? false
   set-default-shape darkness "square"
   set-default-shape predator "hawk"
   create-custom-darkness 1 [ 
     set color black 
     set size 200 
     set hidden? true 
   ]
   create-custom-predator 1 [ 
     set color white 
     set size 4 
     set hidden? true 
   ]
   update-batch-metrics
end

to go
  set ticks (ticks + 1)
  cycle-to-daytime
  ask moths [ 
    assign-genotype-labels 
    set size moth-size 
  ]
  ask predator [ show-predator ]
  eat-moths
  reproduce-moths
  update-batch-metrics
end

;;;;;;;;;;;;;;;;;;;
;; Setup Procedures
;;;;;;;;;;;;;;;;;;;

to make-initial-moths
   create-custom-moths carrying-capacity [
     set size moth-size
     set shape "moth"
     set red-pigment-gene-frequency precision (random-float 11) 1 
     set blue-pigment-gene-frequency precision (random-float 11) 1
     set green-pigment-gene-frequency precision (random-float 11) 1
     setxy (random-float screen-size-x ) (random-float screen-size-y )
     assign-genotype-labels
     set-phenotype-color
   ]
end

to assign-genotype-labels  ;; turtle procedure
   ifelse show-genotype?
    [ set label red-pigment-gene-frequency + " "  
                + green-pigment-gene-frequency + " " 
                + blue-pigment-gene-frequency ]
    [ set label ""]
end

;; convert the genetic representation of gene frequency 
;; into a phenotype using the rgb primitive
to set-phenotype-color  ;; turtle procedure
  set color ( rgb (red-pigment-gene-frequency / 10) 
                  (green-pigment-gene-frequency / 10) 
                  (blue-pigment-gene-frequency / 10) )
end

;;;;;;;;;;;;;;;;;;;;;
;; Runtime Procedures
;;;;;;;;;;;;;;;;;;;;;

to eat-moths
  if mouse-down? [
    let number-here (count moths-at mouse-xcor  mouse-ycor)
    set total-caught (total-caught + number-here)
    ask moths-at mouse-xcor mouse-ycor [die]
    if (number-here > 0) [
      ;;if there was a moth caught here, then cycle to night time
      cycle-to-nighttime
      set caught-in-batch (caught-in-batch + number-here)
    ]  
 ]
end

to reproduce-moths
    if ((count moths) < carrying-capacity) 
    [ ask random-one-of moths [ make-one-offspring ] ]
end

to make-one-offspring ;; turtle procedure
   ;; three possible random mutations can occur, one in each frequency of gene expression
   let red-gene-mutation ((random-float (mutation-step + 1)) - (random-float (mutation-step + 1)))
   let green-gene-mutation ((random-float (mutation-step + 1)) - (random-float (mutation-step + 1)))
   let blue-gene-mutation ((random-float (mutation-step + 1)) - (random-float (mutation-step + 1)))

   hatch 1 [
     set red-pigment-gene-frequency precision (red-pigment-gene-frequency  + red-gene-mutation) 1
     set green-pigment-gene-frequency precision (green-pigment-gene-frequency  + green-gene-mutation) 1
     set blue-pigment-gene-frequency precision (blue-pigment-gene-frequency  + blue-gene-mutation) 1
     if gene-limits? [set-genotype-limits]
     set-phenotype-color
     wander
   ]
end

;; cycling between night and day simply shows and hides a single large turtle that is black to simulate 
;; making the background black and covering the moths in darkness without actually 
;; removing the imported drawing or changing the state of any of the moths
to cycle-to-nighttime
  if day-night? [ 
    ask darkness [ set hidden? false ]
    set night? true
  ]
end

to cycle-to-daytime
  if (day-night? and night?) [
    wait night-wait 
    ask darkness [ set hidden? true ] 
    set night? false 
  ]
end

;; show the hawk shape under the mouse-pointer in the View if predator? is true
to show-predator ;; predator procedure
  ifelse (predator? and mouse-inside?)
    [ set hidden? false setxy mouse-xcor mouse-ycor ]
    [ set hidden? true ]
end

to make-generation
   ask moths [ make-one-offspring ]
end

to wander ;; turtle procedure
   set heading (random-float 360)
   fd (random-float (offspring-distance + 1))
end

;; imposes a threshold limit on gene-frequency.
;; without this genes could drift into negative values or very large values
to set-genotype-limits
   if (red-pigment-gene-frequency < 0) [ set red-pigment-gene-frequency 0 ]
   if (green-pigment-gene-frequency < 0) [ set green-pigment-gene-frequency 0 ]
   if (blue-pigment-gene-frequency < 0) [ set blue-pigment-gene-frequency 0 ]

   if (red-pigment-gene-frequency > 10) [ set red-pigment-gene-frequency 10 ]
   if (green-pigment-gene-frequency > 10) [ set green-pigment-gene-frequency 10 ]
   if (blue-pigment-gene-frequency > 10) [ set blue-pigment-gene-frequency 10 ]
end

to change-environment
    ;; loads an image as a background from the current directory the model was launched from
    import-drawing environment
end

;; a visualization technique to find moths if you are convinced they are not there anymore
;; it allows flashing without actually changing and recalculating the color attribute of the moths
to flash-moths  ;; turtle procedure
  repeat 3 [
    set shape "moth-black"
    wait 0.1
    set shape "moth-white"
    wait 0.1
  ]
  set shape "moth"
end

;;;;;;;;;;;;;;;;;;;;;;
;; Plotting Procedures
;;;;;;;;;;;;;;;;;;;;;;

to update-batch-metrics
  if (caught-in-batch > batch-size) [
    set batch-number (batch-number + 1)
    set caught-in-batch 0
    set interval (ticks - last-ticks)
    set last-ticks ticks
  ]
  do-plots
end

to do-plots
  if total-caught > 0
  [
    set-current-plot "Moths Caught vs. Time"
    set-plot-pen-color black
    plot total-caught
  ]

  set-current-plot "Time to Catch 10 Moths"
  set-plot-pen-color black
  plotxy batch-number interval
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the projects:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS and INTEGRATED SIMULATION AND MODELING ENVIRONMENT.
; The project gratefully acknowledges the support of the
; National Science Foundation (REPP & ROLE programs) -- grant numbers 
; REC #9814682 and REC-0126227.
;
; Copyright 2005 by Uri Wilensky.  Updated 2005.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2005).  NetLogo Bug Hunt Camouflage model.
; http://ccl.northwestern.edu/netlogo/models/BugHuntCamouflage.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2005 Uri Wilensky.  All rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/BugHuntCamouflage
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
350
10
770
451
20
20
10.0
1
10
1
1
1
0
0
0
1

CC-WINDOW
5
513
779
608
Command Center
0

BUTTON
1
10
56
43
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
57
10
112
43
NIL
go
T
1
T
OBSERVER
NIL
NIL

MONITOR
259
209
339
258
NIL
total-caught
0
1

SLIDER
113
10
260
43
carrying-capacity
carrying-capacity
0
100
25
1
1
NIL

SLIDER
1
112
217
145
offspring-distance
offspring-distance
1.0
20.0
5.0
1
1
NIL

CHOOSER
350
454
482
499
environment
environment
"seashore.jpg" "glacier.jpg" "poppyfield.jpg"
1

BUTTON
485
460
618
493
NIL
change-environment
NIL
1
T
OBSERVER
T
NIL

BUTTON
261
10
347
43
flash
flash-moths
NIL
1
T
TURTLE
T
NIL

SWITCH
194
146
347
179
show-genotype?
show-genotype?
1
1
-1000

SLIDER
122
44
224
77
moth-size
moth-size
0.1
10
1.0
0.1
1
NIL

BUTTON
225
44
347
77
make a generation
make-generation\nask moths [assign-genotype-labels]
NIL
1
T
OBSERVER
T
NIL

BUTTON
620
460
727
493
clear background
clear-drawing
NIL
1
T
OBSERVER
T
NIL

SWITCH
218
112
347
145
gene-limits?
gene-limits?
0
1
-1000

SLIDER
1
44
121
77
mutation-step
mutation-step
0
10
1.0
0.1
1
NIL

SWITCH
218
78
346
111
day-night?
day-night?
1
1
-1000

SLIDER
114
78
217
111
night-wait
night-wait
0
10
0.1
0.1
1
NIL

PLOT
14
209
252
348
Moths Caught vs. Time
time
moths
0.0
10.0
0.0
10.0
true
false

SWITCH
1
78
113
111
predator?
predator?
1
1
-1000

PLOT
14
349
252
499
Time to Catch 10 Moths
batch
time
0.0
10.0
0.0
10.0
true
false

@#$#@#$#@
WHAT IS IT?
-----------
A natural/artificial selection model that shows how camouflaging can result very quickly within a population that has a predator hunting it.  For example, green bugs may emerge as the predominant color in a population of bugs in a forest.

When a predator uses color and shape to identify the location of prey in an environment, then the colors and patterns in the environment provide additional selective pressure on the prey.  If some prey tend to blend into the background better, these prey tend to survive longer and reproduce more often.  If this continues over many generations, the distribution of colors in a population may shift to become more camouflaged with the surrounding environment.


HOW IT WORKS
------------
You assume the role of a predator (a bird).  When you press GO, click on moths as fast as you can using your mouse.  You may also keep the mouse button depressed, and move the cursor around the View.

You can watch your consumption by watching the Moths Caught vs. Time graph.  As its slope decreases, you efficiency rate of consumption is decreasing.  Also, the time to catch a batch of moths is plotted in the Time to Catch 10 MOTHS graph.  Try to consume bugs as fast as possible to remove any "deliberation" on your part as a predator.  The camouflaging effect will emerge more clearly, if you aren't taking your time trying to find bugs that typically would be more difficult to find.

Moths have 3 genes that determine their phenotype for color.  One gene is RED-PIGMENT-GENE-FREQUENCY, another is GREEN-PIGMENT-GENE-FREQUENCY, and the last is BLUE-PIGMENT-GENE-FREQUENCY.  The more frequently the gene for a pigment is coded for, the stronger that presence of color is in the overall blend of pigments that results in a single phenotype for coloration (determined by an rgb calculation).

With each bug you eat, a new bug is randomly chosen to reproduce one offspring.  Its offspring's gene-frequency for each of the three gene pigments may be different than the parent (as determined by the MUTATION-STEP slider)

Over time a small population of moths will become harder and harder to detect in the environment (an image file that is loaded into the model).  This is a completely random process that is forced in a particular direction due to selective pressure the user unintentionally exerts on the moths.  Camouflaging emerges from a genetic representation for genes to show color, a selective pressure, and small random changes that accumulate in new offspring in the remaining population.


HOW TO USE IT
-------------
CARRYING-CAPACITY determines the population level on SETUP and how many moths are on the screen at one time when GO is pressed and moths are being eaten.

MUTATION-STEP determines how much the pigment genes could possibly drift from their current values with the next generation.  For example a mutation-step of 1 means that the gene frequency for any of the three pigments could go up 1, down 1 (-1) or not change at all in the offspring.

OFFSPRING-DISTANCE determines how far away an offspring could show up from a parent.  For example, an off-spring distance of 5 means the offspring could be 0, 1, 2, 3, 4, or 5 patches away from the parent.

MOTH-SIZE can be changed during GO or before SETUP to change the size of the shapes for the moths.

GENOTYPE-LIMITS?, limits the maximum value for a gene frequency to be 10 and a minimum value to be 0 when turned on.

SHOW-GENOTYPE? reveals the rgb values for the genes for each moth.

PREDATOR? puts a hawk shape under your mouse pointer if set to true.  With the hawk visible, place the beak of the hawk on the moth to eat it when MOTH-SIZE is 1 or less.  When the hawk is invisible, place the tip of the mouse pointer on the moth to eat it.  When moths are bigger than 1, it is harder to make sure that you are clicking on the center of them.  If you do not click on the center of a moth of size 5, you won't select the moth at that patch location and therefore will not eat the moth.

DAY-NIGHT? will make the screen black after you eat a moth when turned on.  This hides the background and moths to enhance the camouflaging effect for new offspring.  The time it is black is determined by the NIGHT-WAIT slider.

FLASH flashes the moths on the screen in case you are convinced they are no longer there, even though they may simply be very well hidden.

ENVIRONMENT specifies the file name to load as a background on SETUP or on CHANGE-ENVIRONMENT.  The image file should be located in the same directory as the model to load correctly.

MAKE-SINGLE-GENERATION creates one offspring from the existing moths on the screen, without being limited by the CARRYING-CAPACITY.

TOTAL-CAUGHT keeps track of how many moths you have caught.

The plots MOTHS CAUGHT VS TIME and TIME TO CATCH 10 MOTHS plot your progress as a predator.

THINGS TO TRY
-------------
Try this on a user who has not seen the model.  A common response from the user (within about 1 minute of interaction with the model) is "where did the moths all go?"  If you keep playing with the model the user might get better at finding the moths, but if the user keeps trying to catch bugs quickly, even an experienced user will find that the moths will become very hard to find relatively quickly in certain environments.

The DAY-NIGHT? switch was introduced to make memorization of where a new offspring popped up harder to do.  When DAY-NIGHT? is turned off, it is easy to see the sudden appearance of something new in the environment and go after it.  If, however, the environment is hidden briefly (DAY-NIGHT? set to true for some length of time), it harder for the user to make "photographic comparison" of the environment to hunt for new offspring that are better camouflaged.

Sometimes two or more "near background" colors emerge as a predominant feature in a population of moths.  An example of this is the appearance of mostly green and red moths in the poppy field, or dark blue/black and snow blue in the glacier background.  Other times, the saturation of the moths appears to be selected for.  An example of this is a common outcome of "shell colored" moths on the seashore background (e.g. light yellow, light tan, and light blue moths similar to the shells of the seashore).

Larger numbers of moths tend to take longer to start camouflaging.

In environments that have two distinct areas (such as a ground and sky), each with their own patterns and background colors, you might see two distinct populations of different camouflaging outcomes.  Often, while hunting in one area, a user will be surprised to look over at the other area (after they hadn't been paying attention to that area in a while) and notice that now there are a bunch of moths in that background that blend in this new area very well, but whose colors are distinctly different than those that blend into the original area they were hunting in.

Once you reach a point where you are having trouble finding the moths, it is useful to either press FLASH-MOTHS to show you where they are so you can see how they are camouflaged or to press CLEAR-BACKGROUND to let you study their color distribution.


EXTENDING THE MODEL
-------------------
Make this model into a HubNet activity and tap into the natural competition between people to better simulate competition between predators.

What if you could eat a batch of moths (e.g. 5 or 10) before they reproduced?

What if moths reproduced sexually and recombined gene frequencies in their offspring?

What if the position of the moths changed slightly after each day/night cycle?


NETLOGO FEATURES
----------------
IMPORT-DRAWING is the primitive that loads the image into the drawing, which in this case in merely a backdrop.

MOUSE-INSIDE? is the primitive that checks to see if the mouse is within the view.


RELATED MODELS
--------------
Bug Hunt Speeds
Peppered Moths


CREDITS AND REFERENCES
----------------------
Inspired by this BugHunt! Macintosh freeware: http://bcrc.bio.umass.edu/BugHunt/

To refer to this model in academic publications, please use: Wilensky, U. (2005). NetLogo Bug Hunt Camouflage model. http://ccl.northwestern.edu/netlogo/models/BugHuntCamouflage. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2005 Uri Wilensky. All rights reserved. See http://ccl.northwestern.edu/netlogo/models/BugHuntCamouflage for terms of use. 
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

hawk
true
0
Polygon -7500403 true true 166 230 165 255 138 289 158 304 171 304 194 289 165 255
Polygon -16777216 true false 166 222 165 247 143 285 155 295 176 294 189 285 165 247
Polygon -7500403 true true 175 164 174 151 164 144 145 143 154 152 148 171 142 203 150 240 154 262 165 296 175 264 180 240 184 201
Polygon -7500403 true true 153 175 224 159 285 179 271 183 293 199 275 200 286 213 269 213 277 223 263 220 266 227 227 232 152 229
Circle -16777216 true false 159 145 7
Polygon -16777216 true false 144 143 150 148 154 144
Polygon -7500403 true true 168 171 94 158 35 177 51 182 27 197 45 198 34 211 51 211 43 221 57 218 54 225 93 230 168 227
Polygon -7500403 true true 165 214 164 239 142 277 154 287 175 286 188 277 164 239
Polygon -16777216 true false 171 251 164 246 143 270 155 280 176 279 189 270 161 251
Polygon -7500403 true true 165 201 164 226 149 261 154 274 175 273 181 260 164 226
Polygon -16777216 false false 145 142 154 152 149 168 92 158 33 176 49 182 25 196 45 197 33 211 52 211 42 221 57 218 52 225 93 231 147 228 150 240 150 252 149 267 143 271 145 272 141 276 145 281 137 289 150 300 177 300 194 289 184 279 189 275 185 274 188 271 179 262 181 258 176 253 181 240 181 230 228 233 267 227 261 219 277 222 267 213 286 213 275 200 292 198 271 182 285 178 226 159 175 170 174 151 164 144

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

moth
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -7500403 true true 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -7500403 true true 135 90 30
Line -7500403 true 150 105 195 60
Line -7500403 true 150 105 105 60

moth-black
true
0
Polygon -16777216 true false 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -16777216 true false 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -16777216 true false 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -16777216 true false 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

moth-white
true
0
Polygon -1 true false 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -1 true false 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -1 true false 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -1 true false 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -1 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -1 true false 135 90 30
Line -1 false 150 105 195 60
Line -1 false 150 105 105 60

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
