breeds [ bugs predator ]

bugs-own [
    speed 
    detects-predator-within
  ]

globals [
    total-caught 
    total-6-caught 
    total-5-caught 
    total-4-caught 
    total-3-caught 
    total-2-caught 
    total-1-caught
    time 
    time-last-caught 
    time-this-caught 
    total-frequency-of-catching
    changed-color-map? 
    old-color-map 
    histogram-interval-size
  ]

to setup
  ca
  set total-caught 0
  set time 0
  set histogram-interval-size 1
  set old-color-map speed-color-map
  set changed-color-map? false
  ask patches [ set pcolor white ]
  create-custom-bugs number-bugs-each-speed [ set speed 1 ]
  create-custom-bugs number-bugs-each-speed [ set speed 2 ]
  create-custom-bugs number-bugs-each-speed [ set speed 3 ]
  create-custom-bugs number-bugs-each-speed [ set speed 4 ]
  create-custom-bugs number-bugs-each-speed [ set speed 5 ]
  create-custom-bugs number-bugs-each-speed [ set speed 6 ]
  ask bugs [
    set shape "bug"
    setxy random-float screen-size-x random-float screen-size-y
  ]
  create-custom-predator 1 [ set hidden? true ]
  ask bugs [ set-colors ]
  do-plots
end

to go
  let bugs-remaining 0
  set time (time + 1)
  check-color-map-change
  check-caught
  move-predator
  move-bugs
  do-plots
  set bugs-remaining (count bugs)
  set changed-color-map? false
end

;;;;;;;;;;;;;;;;;;;;;
;; Runtime Procedures
;;;;;;;;;;;;;;;;;;;;;

to move-bugs
  let predator-agent one-of predator
  let heading-toward 0
  let distance-from-predator nobody
  ask bugs [
    set distance-from-predator distance-nowrap predator-agent
    ;; if bugs are close enough to see the predator ( which is the mouse cursor ), 
    ;; bugs turn away 
    if (mouse-inside? and distance-from-predator < detects-predator-within)
    [
      set heading-toward towards predator-agent 
      set heading (- heading-toward)
    ]
    fd (speed * speed-factor)
  ]
end

to move-predator
   if (mouse-inside?)
    [ ask predator [ setxy mouse-xcor mouse-ycor ] ]
end

to check-caught
  let speed-of-caught 0
  let period-to-catch 0
  ;; the mouse may move while we are doing calculations
  ;; so keep track of the current mouse position so
  ;; we do all the calculations with the same numbers
  let snap-mouse-xcor mouse-xcor
  let snap-mouse-ycor mouse-ycor
  if mouse-down? and mouse-inside? [
    if (any? bugs-at snap-mouse-xcor snap-mouse-ycor) [
      set total-caught (total-caught + 1)
      set time-last-caught time-this-caught
      set time-this-caught time
      set period-to-catch (time-this-caught - time-last-caught)
      set total-frequency-of-catching (total-frequency-of-catching + (1 / period-to-catch))
      reproduce-one
      ;; eat only one of the bugs at the mouse location
      ask one-of bugs-at snap-mouse-xcor snap-mouse-ycor [
        set speed-of-caught speed
        if (speed-of-caught = 1) [ set total-6-caught (total-6-caught + 1) ]
        if (speed-of-caught = 2) [ set total-5-caught (total-5-caught + 1) ]
        if (speed-of-caught = 3) [ set total-4-caught (total-4-caught + 1) ]
        if (speed-of-caught = 4) [ set total-3-caught (total-3-caught + 1) ]
        if (speed-of-caught = 5) [ set total-2-caught (total-2-caught + 1) ]
        if (speed-of-caught = 6) [ set total-1-caught (total-1-caught + 1) ]
        die
      ]
    ]
  ]
end

to reproduce-one
   ask random-one-of bugs [
     hatch 1 [set heading (random-float 360)]
   ]
end

;; apply color map change only once each time 
;; a new value for speed-color-map is selected
to check-color-map-change  
  if (old-color-map != speed-color-map) [
    set old-color-map speed-color-map
    set changed-color-map? true
    ask bugs [ set-colors ]
  ]
end

to set-colors ;; turtle procedure
    if (speed-color-map = "all green") [ set color green ]
    if (speed-color-map = "blue red") [ recolor-twocolor ]
    if (speed-color-map = "violet shades") [ recolor-shade  ]
    if (speed-color-map = "rainbow") [ recolor-rainbow ]
end

to recolor-twocolor ;; turtle procedure
   ifelse (speed <= 3) 
   [ set color blue ]
   [ set color red ]
end

to recolor-shade ;; turtle procedure
  set color (112 + speed )
end

to recolor-rainbow ;; turtle procedure
   if (speed = 6) [ set color red ]
   if (speed = 5) [ set color orange ]
   if (speed = 4) [ set color (yellow - 1) ]
   if (speed = 3) [ set color green ]
   if (speed = 2) [ set color blue ]
   if (speed = 1) [ set color violet ]
end

;;;;;;;;;;;;;;;;;;;;;;
;; Plotting Procedures
;;;;;;;;;;;;;;;;;;;;;;

to do-plots
  set-current-plot "Bugs Caught vs. Time"
  plot-caught

  set-current-plot "Number of Bugs vs. Time"
  plot-populations

  set-current-plot "Consumption Rate vs. Time"
  if (total-caught > 0)
   [ plotxy time (total-frequency-of-catching / total-caught) ]

  set-current-plot "Frequency of Bugs"
  plot-histograms
end

to plot-caught
  set-current-plot-pen "speed=1"
  plotxy time total-1-caught

  set-current-plot-pen "speed=2"
  plotxy time total-2-caught

  set-current-plot-pen "speed=3"
  plotxy time total-3-caught

  set-current-plot-pen "speed=4"
  plotxy time total-4-caught

  set-current-plot-pen "speed=5"
  plotxy time total-5-caught

  set-current-plot-pen "speed=6"
  plotxy time total-6-caught
end

to plot-populations
  set-current-plot-pen "speed=1"
  plot (count bugs with [ speed = 1 ])

  set-current-plot-pen "speed=2"
  plot (count bugs with [ speed = 2 ])

  set-current-plot-pen "speed=3"
  plot (count bugs with [ speed = 3 ])

  set-current-plot-pen "speed=4"
  plot (count bugs with [ speed = 4 ])

  set-current-plot-pen "speed=5"
  plot (count bugs with [ speed = 5 ])

  set-current-plot-pen "speed=6"
  plot (count bugs with [ speed = 6 ])
end

to plot-histograms
  set-histogram-num-bars 8
  set-current-plot-pen "pen1"
  set-plot-pen-interval histogram-interval-size
  histogram-from bugs with [ speed = 1 ] [ speed ]

  set-histogram-num-bars 8
  set-current-plot-pen "pen2"
  set-plot-pen-interval histogram-interval-size
  histogram-from bugs with [ speed = 2 ] [ speed ]

  set-histogram-num-bars 8
  set-current-plot-pen "pen3"
  set-plot-pen-interval histogram-interval-size
  histogram-from bugs with [ speed = 3 ] [ speed ]

  set-histogram-num-bars 8
  set-current-plot-pen "pen4"
  set-plot-pen-interval histogram-interval-size
  histogram-from bugs with [ speed = 4 ] [ speed ]

  set-histogram-num-bars 8
  set-current-plot-pen "pen5"
  set-plot-pen-interval histogram-interval-size
  histogram-from bugs with [ speed = 5 ] [ speed ]

  set-histogram-num-bars 8
  set-current-plot-pen "pen6"
  set-plot-pen-interval histogram-interval-size
  histogram-from bugs with [ speed = 6 ] [ speed ]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the projects:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS and INTEGRATED SIMULATION AND MODELING ENVIRONMENT.
; The project gratefully acknowledges the support of the
; National Science Foundation (REPP & ROLE programs) -- grant numbers 
; REC #9814682 and REC-0126227.
;
; Copyright 2005 by Uri Wilensky.  Updated 2005.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2005).  NetLogo Bug Hunt Speeds model.
; http://ccl.northwestern.edu/netlogo/models/BugHuntSpeeds.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2005 Uri Wilensky.  All rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/BugHuntSpeeds
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
431
10
781
501
8
11
20.0
1
10
1
1
1
0
1
1
1

CC-WINDOW
5
515
790
610
Command Center
0

MONITOR
301
38
387
87
total caught
total-caught
0
1

BUTTON
6
38
67
71
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
69
38
130
71
NIL
go
T
1
T
OBSERVER
T
NIL

PLOT
214
151
428
326
Bugs Caught vs. Time
time
time
0.0
1000.0
0.0
10.0
true
false
PENS
"total" 1.0 0 -16777216 true
"speed=1" 1.0 0 -8630108 true
"speed=2" 1.0 0 -13345367 true
"speed=3" 1.0 0 -10899396 true
"speed=4" 1.0 0 -3355648 true
"speed=5" 1.0 0 -955883 true
"speed=6" 1.0 0 -2674135 true

PLOT
3
325
215
501
Frequency of Bugs
speeds
frequency
0.0
8.0
0.0
50.0
true
false
PENS
"default" 0.1 1 -16777216 true
"pen1" 0.1 1 -8630108 true
"pen2" 0.1 1 -13345367 true
"pen3" 0.1 1 -10899396 true
"pen4" 0.1 1 -3355648 true
"pen5" 0.1 1 -955883 true
"pen6" 0.1 1 -2674135 true

CHOOSER
276
92
388
137
speed-color-map
speed-color-map
"all green" "rainbow" "blue red" "violet shades"
0

SLIDER
6
73
131
106
speed-factor
speed-factor
0.01
0.4
0.04
0.01
1
NIL

PLOT
3
151
215
326
Consumption Rate vs. Time
time
rate
0.0
1000.0
0.0
0.5
true
false

PLOT
214
325
428
501
Number of Bugs vs. Time
time
number
0.0
1000.0
0.0
20.0
true
false
PENS
"total" 1.0 0 -16777216 true
"speed=1" 1.0 0 -8630108 true
"speed=2" 1.0 0 -13345367 true
"speed=3" 1.0 0 -10899396 true
"speed=4" 1.0 0 -3355648 true
"speed=5" 1.0 0 -955883 true
"speed=6" 1.0 0 -2674135 true

MONITOR
196
38
297
87
bugs on screen
(count bugs)
0
1

SLIDER
6
108
191
141
number-bugs-each-speed
number-bugs-each-speed
2
50
9
1
1
NIL

MONITOR
196
91
271
140
NIL
time
0
1

@#$#@#$#@
WHAT IS IT?
-----------
This is a natural/artificial selection model that shows the result of two competing forces on natural selection of the speed of prey.  Which force dominates depends on the behavior of predators.

One force is that predators that chase prey, tend to catch slower moving prey more often, thereby selecting for prey that are faster over many generations of offspring.

Another force is that predators who wait for their prey without moving, tend to catch prey that are moving faster more often, thereby selecting for prey that are slower over many generations of offspring.

By also adjusting whether bugs try to avoid the predator and the predictability of their motion, a different one of these competing forces will tend to dominate the selective pressure on the population.


HOW IT WORKS
------------
You assume the role of a predator amongst a population of bugs.  To begin your pursuit of bugs as a predator, press SETUP to create a population of bugs, determined by the six times the NUMBER-BUGS-EACH-SPEED slider.  These bugs that are created are randomly distributed around the view and assigned a speed.

When you press GO the bugs begin to move at their designated speeds.  As they move around, try to eat as many bugs as fast as you can by clicking on them in the view.  Alternatively, you may hold the mouse button down and move the mouse pointer over the bugs in the view.

The six different speeds that a bug might move at are distributed amongst six different sub-populations of the bugs.  These speeds are genetically inherited.  With each bug you eat, a new bug is randomly chosen from the population to reproduce one offspring.  This bug's offspring is an exact duplicate of the parent (in terms of its speed, orientation, and location).  The creation of this new offspring keeps the overall number population of the bugs on the screen constant.

Initially there are equal numbers of each sub-population of bug (e.g. ten bugs at each of the 6 speeds).  Over time, however, as you eat bugs, the distribution of the bugs will change as shown in the "Frequency of bugs" Histogram and the "Number of Bugs vs. Time" graph.  In the histogram, you might see the distribution shift to the left (showing that more slow bugs are surviving) or to the right (showing that more fast bugs are surviving).  Sometimes one sub-population of a single speed of bug will be exterminated.  At this point, no other bugs of this speed can be created in the population.


HOW TO USE IT
-------------
NUMBER-BUGS-EACH-SPEED is the number of bugs you start with in each of the six sub-populations.  The overall population of bugs is determined by multiplying this value by 6.

SPEED-FACTOR is an overall speed coefficient to use to speed up or slow down all the bugs, without making their motion jerky.  This is an important slider to adjust to compensate for different speed platforms the model might run on.

SPEED-COLOR-MAP settings help you apply color visualization to the speed of the bugs.  The "all green" setting does not show a different color for each bug based on its speed".  Keeping the color settings switched to something besides "all green" can tend to result in the predator (the user) unconsciously selecting bugs based on color instead of speed.
The "rainbow" setting shows 6 distinct colors for the 6 different speeds a bug might have.  These color settings correspond to the plot pen colors in the graphs.
The "blue red" setting shows the lower half of the speeds of the starting population as blue, and the upper half as red.
The "purple shades" setting shows a gradient of dark purple to light purple for slow to fast bug speed.


THINGS TO NOTICE
----------------
This histogram tends to shift right if you assume the role of chasing easy prey.

This histogram tends to shift left if you assume the role of waiting for prey come to you.  (The same effect is achieved with moving the mouse around the screen randomly)


THINGS TO TRY
-------------
Wait in one location for the bugs to come to you by placing the cursor in one location and holding down the mouse button.  All bugs that run into you will be eaten.

Chase bugs around trying to catch the bug nearest you at any one time by holding the mouse button down and moving the cursor around the screen after the nearest bug.  


EXTENDING THE MODEL
-------------------
The Bug Hunt Speeds model includes bugs that wander in a path that is not a straight line and allow you to set whether the bugs avoid the predator.  Both of these characteristics as well as speed are inheritable characteristics in the Bug Hunt Scurry model.

A HubNet version of the model with adjustable starting populations of bugs would help show what happens when two or more competitors assume similar vs. different hunting strategies on the same population at the same time.


RELATED MODELS
--------------
Bug Hunt Camouflage


CREDITS AND REFERENCES
----------------------
Inspired by EvoDots software:
http://faculty.washington.edu/~herronjc/SoftwareFolder/EvoDots.html

To refer to this model in academic publications, please use: Wilensky, U. (2005). NetLogo Bug Hunt Speeds model. http://ccl.northwestern.edu/netlogo/models/BugHuntSpeeds. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2005 Uri Wilensky. All rights reserved. See http://ccl.northwestern.edu/netlogo/models/BugHuntSpeeds for terms of use. 
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0beta6
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
