globals [
  ;;patches that generate the singles and pairs
  generator-patches

  ;;offset variable
  pxcor-offset

  ;;list of outcomes of singles and pairs, to plot in a histogram
  single-die-outcomes-list
  dice-pair-outcomes-list

  ;;columns colors
  column1-color
  column2-color
  mid-column-color

  #runs
]

breeds [
  pairs
  singles
]

patches-own [
  sum-to-match ;; grabs pairs
  num-to-match ;; grabs singles
  offset-pxcor
]

pairs-own [
  sum-of-roll
]

;;; pairs and singles both own these
turtles-own [
  die-value
  moving?
]

to startup
  set #runs 0
end

to setup
  ca
  set column1-color 5
  set column2-color 34
  set mid-column-color 8
  set pxcor-offset -8

  set single-die-outcomes-list []
  set dice-pair-outcomes-list []

  ;;sets the offset xcor value for patches
  ask patches [
    set offset-pxcor pxcor - pxcor-offset
  ]
  ;;color right part of display, set the sum-to-match
  ask patches with [ offset-pxcor > 4 ] [
    set sum-to-match int ((offset-pxcor - 1) / 2)
    ifelse sum-to-match mod 2 = 0 [ set pcolor column1-color ] [ set pcolor column2-color ]
  ]
  ;;color left part of display, set the sum-to-match
  ask patches with [ offset-pxcor < -4 ] [
    set num-to-match screen-edge-x + pxcor + 1
    ifelse num-to-match mod 2 = 0 [ set pcolor column1-color ] [ set pcolor column2-color ]
  ]

  ;;sets the generator patch values
  set generator-patches patches with [
    pycor = screen-edge-y and
      ( offset-pxcor = -1 or
        offset-pxcor = 0 )
  ]

  ;;color middle columns
  ask patches with [ abs offset-pxcor < 5 ] [
    set pcolor mid-column-color
  ]
end

to go
  let moving-turtles turtles with [ moving? ]
  ;;if no dice currently moving, bump down necessary columns
  ;;(or stop if display is full and stop-at-top? is true), then create a new pair of dice
  ifelse not any? moving-turtles [
    while [ any? turtles with [ (pycor = screen-edge-y) and (offset-pxcor > 4) ] ] [
      if stop-at-top? [ stop ]
      bump-down-pairs
    ]
    while [ any? turtles with [ (pycor = screen-edge-y - 2) and (offset-pxcor < -4)] ] [
       bump-down-singles
    ]
      generate-roll
  ]
  ;;if there are dice moving, proceed with their path
  [
    go-pairs
    go-singles
  ]
end

;;creates a new pair of dice (both singles and pairs)
to generate-roll
  ;;ask each generator patch to create a pair (one die, with breed pair),
  ;;with a random value
  ask generator-patches [
    sprout-pairs 1 [
      set color white
      set die-value 1 + random 6
      set shape "die " + die-value
      set moving? true
      set heading 90
    ]
  ]

  ;;set the sum variable of the pairs
  ask pairs with [ moving? ] [
    set sum-of-roll sum values-from pairs with [ moving? ] [ die-value ]
  ]

  ;;clones the pairs to make the singles with the pair's value
  ask generator-patches [
    sprout-singles 1 [
      set color white
      set die-value die-value-of one-of other-turtles-here
      set shape "die " + die-value
      set moving? true
      set heading 270
    ]
  ]

  set #runs (#runs + 1)
end

;;moves the pairs a step either to the correct column, down the column or stops the pair
to go-pairs
  let pair-value -1
  let moving-pairs pairs with [ moving? ]
  ;;if a pair is not at the correct column, move on
  ifelse any? moving-pairs with [ sum-of-roll != sum-to-match ] [
    ask moving-pairs [ fd 1 ]
  ]

  [
    ask moving-pairs [
      ;;move down
      set heading 180

      ;;if at the bottom of the screen, check it the pair should
      ;;go "under water"
      if pycor = (- screen-edge-y)
      [
        set pair-value sum-of-roll-of one-of moving-pairs
        should-I-die?-pairs
      ]

      ;;if reaches the bottom (and the pair is "above water") or there is a pair in front, stops
      ifelse pycor = (- screen-edge-y ) or
             any? value-from patch-ahead 1 [ pairs-here with [ not moving? ] ]
      [
        set pair-value sum-of-roll-of one-of moving-pairs
        set moving? false
      ]
      ;;if nothing ahead of the pair and it is not at the bottom, it moves one down the column
      [
        fd 1
      ]
    ]
  ]

  if pair-value != -1 [ update-pair-outcomes pair-value ]
end

;;moves the singles a step either to the correct column, down the column or stops the single
to go-singles
  let moving-singles singles with [ moving? ]
  ;;if a single is not at the correct column, move on
  ask moving-singles [
    without-interruption
    [
      if die-value = num-to-match [
        ;;move down
        set heading 180
      ]

      ;;if at the bottom of the screen, check it the single should
      ;;go "under water"
      if pycor = (- screen-edge-y) [ should-I-die?-singles ]

      ;;if reaches the bottom (and the pair is "above water") or there is a single in front, stops
      ifelse pycor = (- screen-edge-y ) or
             any? value-from patch-ahead 1 [ singles-here with [ not moving? ] ]
      [
        update-single-outcomes die-value
        set moving? false
      ] [
        ;;if nothing ahead of the single and it is not at the bottom, it moves one down the column
        fd 1
      ]
    ]
  ]
end

to should-I-die?-singles  ;;turtle procedure
  if single-die-outcomes-list != []
  [
    ;; determines if the dice-pair's column is tall enough to be seen in the truncated view
    let mode-of-dice-list first modes single-die-outcomes-list
    let height-of-tallest-column length filter [ ? = mode-of-dice-list ] single-die-outcomes-list
    let height-of-my-column length filter [ ? = die-value ] single-die-outcomes-list
    let my-diff ( height-of-tallest-column - height-of-my-column )

    if my-diff >= screen-size-y - 2
    [
      ;;updates the list, plots
      set single-die-outcomes-list (lput die-value single-die-outcomes-list)
      update-single-outcomes die-value
      die
    ]
  ]
end

to should-I-die?-pairs ;; turtle procedure
  if dice-pair-outcomes-list != []
  [
    ;; determines if the dice-pair's column is tall enough to be seen in the truncated view
    let mode-of-pairs-list first modes dice-pair-outcomes-list
    let height-of-tallest-column length filter [ ? = mode-of-pairs-list ] dice-pair-outcomes-list
    let height-of-my-column length filter [ ? = sum-of-roll ] dice-pair-outcomes-list
    let my-diff ( height-of-tallest-column - height-of-my-column )
    if my-diff >= screen-size-y - 1 [die]
  ]
end

;;moves each pair ahead one, kills ones that are on bottom of screen
to bump-down-pairs
  ask turtles with [ offset-pxcor > 4 ] [
    ifelse ycor = (- screen-edge-y)
      [ die ]
      [ fd 1 ]
    ]
end

;;moves each single ahead one, kills ones that are on bottom of screen
to bump-down-singles
  ask turtles with [ offset-pxcor < -4 ] [
    ifelse ycor = (- screen-edge-y)
      [ die ]
      [ fd 1 ]
    ]
end

;;updates the histograms
to update-plots
  set-current-plot "Single Die"
  histogram-list single-die-outcomes-list
  set-current-plot "Dice Pair Total"
  histogram-list dice-pair-outcomes-list
end

;;adds the die-value to the list of outcomes of singles, update plots
to update-single-outcomes [ value-of-die ]
  set single-die-outcomes-list sentence
    single-die-outcomes-list value-of-die
  update-plots
end

;;adds the sum-of-roll to the list of outcomes of pairs, updates plots
to update-pair-outcomes [ value-of-sum ]
  set dice-pair-outcomes-list sentence
    dice-pair-outcomes-list value-of-sum
  update-plots
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the projects:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS and INTEGRATED SIMULATION AND MODELING ENVIRONMENT.
; The project gratefully acknowledges the support of the
; National Science Foundation (REPP & ROLE programs) -- grant numbers
; REC #9814682 and REC-0126227.
;
; Copyright 2004 by Uri Wilensky.  Updated 2005.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Abrahamson, D. & Wilensky, U. (2004).  NetLogo Dice Stalagmite model.
; http://ccl.northwestern.edu/netlogo/models/DiceStalagmite.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2004 Uri Wilensky.  All rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/DiceStalagmite
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
177
39
557
580
18
25
10.0
1
10
1
1
1
0
1
1
1

CC-WINDOW
5
607
792
702
Command Center
0

BUTTON
11
10
90
43
Setup
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
92
10
171
43
Go
go
T
1
T
OBSERVER
T
NIL

PLOT
11
48
171
590
Single Die
Die Value
Count
1.0
7.0
0.0
51.0
true
false
PENS
"default" 1.0 1 -16777216 true

PLOT
562
49
783
593
Dice Pair Total
Dice Total
Count
2.0
13.0
0.0
51.0
true
false
PENS
"default" 1.0 1 -16777216 true

MONITOR
249
85
327
134
Total Pairs
#runs
3
1

SWITCH
374
4
498
37
stop-at-top?
stop-at-top?
0
1
-1000

@#$#@#$#@
WHAT IS IT?
-----------
Dice Stalagmite is a model for thinking about the relations between independent and dependent random events. A random pair of dice is produced, and then these dice fall into columns in two bar charts, one recording the dice as independent outcomes, and the other, as a single compound event of these two outcomes. Because the columns grow from the bottom up, we call this a "stalagmite."

Different typical distributions emerge: the independent-event bar chart is "flat" (equally distributed) whereas the dependent-event bar chart is "triangular" (it does not quite approach a normal distribution, because there are only two compound outcomes).

This model is a part of the ProbLab curriculum.  The ProbLab curriculum is currently under development at the CCL.  For more information about the ProbLab Curriculum please refer to http://ccl.northwestern.edu/curriculum/ProbLab/.


HOW IT WORKS
------------
In this model, outcomes from two independent events -- the rolling of "Die A" and "Die B" -- are represented in two different ways.  On the left of the display, you see these outcome pairs plotted as individual events -- this representation ignores the fact that the two dice were rolled as a pair and just stacks each die outcome in its respective column within the picture bar chart.  On the right of the display, you see the same two dice accumulating as pairs.

Note that the bar chart on the left has only six columns: 1, 2, ..., 6 -- one for each possible die face.  The chart on the right has 11 columns -- 2, 3, ..., 12 -- because it groups outcomes by total, and there are eleven possible totals (these columns are broader, because they must accommodate two dice).

The random outcomes first appear in the top middle of the display.  Then they are copied, so that we have one copy for the individual dice and one copy for the pair of dice. The individual dice then move to the left towards the single-die bar chart, and the pair moves to the right towards the dice-pair bar chart.  On each side, the dice find their column -- the column that matches their value, either as individuals (on the left) or as a pair (on the right) -- and descend down this column until they reach either the bottom of the column or another previously stacked event.

When the model is run, the bar chart on the right never reaches the top before the bar chart on the left. (Why?)  A procedure "bumps" the left bar chart down by one row so as to leave more room for the bars to grow.  This allows for the bar chart on the right to grow further and take on its typical (triangular) shape.


HOW TO USE IT
-------------
Switches:
STOP-AT-TOP? -- if 'On', stops the model when the right side of the display bar chart (the dice totals) has reached the top.

Buttons:
SETUP -- prepares the model for running.

GO -- runs the model.  In a single run of GO, a random pair of dice appears and then finds its place in the charts.  Also, the plots are updated.

Monitors:
TOTAL PAIRS -- displays the number of pairs that have been produced (the number of samples).

Plots:
SINGLE DIE -- plots the number of occurrences of each die-number (1-6).

DICE PAIR TOTAL -- plots the number of occurrences of each die-total (2-12).


PEDAGOGICAL NOTE
------------
As in other ProbLab activities, in Dice Stalagmite, we care for exploring relations between the anticipated frequency distribution (the relative probabilities), which we determine through combinatorial analysis, and the outcome distribution we receive in computer-based simulations of probability experiments. To facilitate the exploration of the relationship between such theoretical and empirical work, we build tools that bridge between them. These bridging tools have characteristics of both the theoretical and empirical work. Specifically, we structure our combinatorial spaces in formats that resemble outcome distributions, and structure our experiments so as to sustain the raw data (not just graphs representing the data). The "picture bar chart" of the combinatorial space of dice-pair totals can be found with the ProbLab materials.

Beside each bar chart -- the 'dependent' and the 'independent' -- there is a histogram that represents the data correspondingly. Whereas the bar charts stack the outcomes so as to sustain the images of the discrete events (the "raw data" themselves), the histograms grow in continuous columns (without partition lines). Twinning each picture bar chart with its respective histogram may help students both to understand the histograms and to shift from additive interpretation of the columns in the picture bar chart (focusing on differences between heights of columns) to a multiplicative interpretation of the bar chart (focusing on the proportions of the column heights).

In a classroom, students should work with the triangular combinatorial space they created (all 36 different possible outcomes of a dice pair that are arranged in a bar chart). Discussion should focus on the relation between the theoretical and empirical distribution, that is, between the combinatorial space and the distribution of random outcomes. Why is it that they are similar?


THINGS TO NOTICE
----------------
Note the shape of the outcomes in the right-hand bar chart. It is triangular. What does this mean? Specifically, if each event is random and independent, why are we getting a shape that was not random (we keep getting the same shape)? How can this randomness and determinism coexist? The bar chart on the left hones this discussion, because, from run to run, it is basically a "flat" distribution -- for instance, you can never predict with certainty which die column will be first to reach the top.

If the model runs long enough -- for instance, if STOP-AT-TOP? is set to 'Off,' you will notice that some columns in the picture bar chart on the left vanish. That is, you will see a die descending to the bottom of its column and "going below sea level" so it is no longer visible. What happens is that this die's column is now too short to appear in the display. It might grow tall enough later to come back in, though. Or not. Meanwhile, the histogram in the plot keeps all of its columns, so you can keep comparing between them.


THINGS TO TRY
-------------
How many samples are needed until the dice-pair bar chart reaches the top? Is this number constant? How much does it vary?

What is the biggest vertical gap between columns in the single-die bar chart? Does the gap get larger or smaller the more you run the model? Does any particular column win more often than others?

Which column in the dice-pair bar chart gets to the top first most often?


EXTENDING THE MODEL
-------------------
Currently, the model sums two dice at a time.  An interesting idea would be to extend this model to have a sum of three dice, instead of 2.  There would be more columns for the different dice-totals (how many?).  How would this change affect the dice-total distribution?


NETLOGO FEATURES
----------------
Sometimes the model developer finds it useful to change the coordinate system of the display for convenience.  In this model, we made a new variable for patches that held its "offset x-value."  In fact, the model does not use the "pxcor" variable of the patches at all.  In effect, the whole model shifts.  This is convenient, because we put the 0 of the model at a point between the two displays, so when the model refers to the points on the screen, it does it in general terms of that 0, instead of the display's "pxcor" 0.  This technique also allows for easier extension of the model.


RELATED MODELS
--------------
Dice Stalagmite uses the same basic metaphor as the ProbLab model 9-Block Stalagmite.  In that model, a random 9-block or 4-block is selected from a sample space.  Then, the block finds is correct column, according to the number of green squares in the block, and stacks up in that column.

The idea of juxtaposing two or more different representations of the same running data is used in several ProbLab models, such as Prob Graph Basic or Random Combinations and Permutations.

Dice are also used in the ProbLab model Dice for generating a distribution of random outcomes.

The Galton Box model also features raw data that descend and stack up in columns.


CREDITS AND REFERENCES
----------------------
This model is a part of the ProbLab curriculum. The ProbLab Curriculum is currently under development at Northwestern's Center for Connected Learning and Computer-Based Modeling. . For more information about the ProbLab Curriculum please refer to http://ccl.northwestern.edu/curriculum/ProbLab/.

Thanks to Josh Unterman for building the original version of this model. Thanks to Steve Gorodetskiy for his contribution to the design of this model.        

To refer to this model in academic publications, please use: Abrahamson, D. & Wilensky, U. (2004). NetLogo Sample Stalgmite model. http://ccl.northwestern.edu/netlogo/models/DiceStalagmite. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2004 Uri Wilensky. All rights reserved. See http://ccl.northwestern.edu/netlogo/models/DiceStalagmite for terms of use.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

die 1
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 129 129 42

die 2
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 69 69 42
Circle -16777216 true false 189 189 42

die 3
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 69 69 42
Circle -16777216 true false 129 129 42
Circle -16777216 true false 189 189 42

die 4
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 69 69 42
Circle -16777216 true false 69 189 42
Circle -16777216 true false 189 69 42
Circle -16777216 true false 189 189 42

die 5
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 69 69 42
Circle -16777216 true false 129 129 42
Circle -16777216 true false 69 189 42
Circle -16777216 true false 189 69 42
Circle -16777216 true false 189 189 42

die 6
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 84 69 42
Circle -16777216 true false 84 129 42
Circle -16777216 true false 84 189 42
Circle -16777216 true false 174 69 42
Circle -16777216 true false 174 129 42
Circle -16777216 true false 174 189 42

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0beta5
@#$#@#$#@
setup repeat 20000 [ go ]
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
