globals
[
    ratio-brewer
    ratio-wholesaler
    ratio-retailer
    user-numbers-to-names
    retailers
    brewers
    wholesalers
    day

    brewer-purchase-cost
    brewer-sale-price
    brewer-storage-cost
]

breeds[ players ]

players-own
[
    user-id ; used to communicate with the client
    clients 
    supplier
    inventory
    waiting-list
    player-type  ; "retailer", "brewer", "wholesaler"
    last-shipment
    last-week-demand 
    next-order    
    bank-account
    revenue-not-gotten
    brewery-list ; used only by brewers
]

;;;;;;;;;;;;;;;;;;;;;;
;; Setup Procedures ;;
;;;;;;;;;;;;;;;;;;;;;;

to startup
  hubnet-set-client-interface "COMPUTER" [ "clients/Beer Game Alternate 2 client.nlogo" ]
  hubnet-reset
  setup
end

to setup
  ca
  set ratio-brewer 1
  set ratio-wholesaler 2
  set ratio-retailer 6
  set user-numbers-to-names []
  set day 0
  set brewer-purchase-cost .25
  set brewer-sale-price .6
  set brewer-storage-cost .10
end

to reset
    set day 0
    ask players [ reset-player-values ]
end

;;;;;;;;;;;;;;;;;;;;;;;;
;; Runtime Procedures ;;
;;;;;;;;;;;;;;;;;;;;;;;;

to go
  listen-clients
  ask players [ send-info-to-client ]

  every 2
  [
    set day day + 1
    process-orders
  ]
end

to process-orders
  if ( day > 6 ) and (current-day day  = "Friday") ; the day the supppliers receive new orders
  [
    ask players with [ ( player-type = "wholesaler" ) or ( player-type = "brewer" ) ]
    [ set last-week-demand 0 ]
  ]
  if (day > -1 ) and (current-day day  = "Tuesday") ; the day the supppliers receive new orders
  [
    ask players [ set last-shipment 0 ]
  ]
  ask players [ pump ]
end

to-report represent-day [ d ] ;;convert from a day number to a pretty string
    report (current-day d) + " of Week " + (current-week d)
end

to-report current-day [ d ]
    if d mod 7 = 0
    [ report "Sunday" ]
    if d mod 7 = 1
    [ report "Monday" ]
    if d mod 7 = 2
    [ report "Tuesday" ]
    if d mod 7 = 3
    [ report "Wednesday" ]
    if d mod 7 = 4
    [ report "Thursday" ]
    ifelse d mod 7 = 5
    [ report "Friday" ]
    [ report "Saturday" ]
end

to-report current-week [ d ]
  report floor (1 + (d / 7))
end 

;;;;;;;;;;;;;;;;;;;;;;;
;; Player Procedures ;;
;;;;;;;;;;;;;;;;;;;;;;;

to add-new-client [new-client]
    set clients fput new-client clients
end 

to remove-client [ client ]
    set clients remove client clients
end 

;; if there is inventory leftover
to charge-for-storage
  set bank-account ( bank-account - (inventory * role-coefficient player-type) * brewer-storage-cost)
end

to place-order [ order-units ]
    ifelse player-type = "brewer"
    [
        without-interruption
        [
            set brewery-list lput order-units brewery-list
        ]
    ]
    [
        let new-order list user-id order-units
        ask supplier 
        [
            without-interruption
            [
                let temp-list lput new-order first reverse waiting-list
                set waiting-list replace-item ( (length waiting-list) - 1)  waiting-list temp-list
                set last-week-demand last-week-demand + order-units ;update what will be the suppliers demand
            ]
            ;sendInfoToClient
        ]
    ]
end

to pump
  if current-day day = "Tuesday" [ charge-for-storage ]

  if player-type = "retailer"
  [
    ifelse (inventory > 0) and (consumer-demand > 0)
    [
      ifelse (consumer-demand < inventory)
      [
        set bank-account (bank-account + (4 * brewer-sale-price * consumer-demand))
        set inventory (inventory - consumer-demand)
      ]
      [
         if (consumer-demand > inventory)
         [
           set bank-account (bank-account + (4 * brewer-sale-price * inventory) )
           set revenue-not-gotten revenue-not-gotten + (4 * brewer-sale-price * abs (consumer-demand - inventory))
           set inventory 0
         ]
      ]
    ]
   [
      if inventory = 0
      [
        set revenue-not-gotten revenue-not-gotten + (4 * brewer-sale-price * (consumer-demand))
      ]
   ]
   if current-day day = "Friday"
   [
     place-order next-order
   ]
 ]
    if player-type = "wholesaler"
    [
        if current-day day = "Tuesday"
        [
            supply-downstream
        ]
        if current-day day = "Friday"
        [
            place-order next-order
        ]
    ]
    if player-type = "brewer"
    [
        if current-day day = "Tuesday"
        [
            without-interruption
            [
                set inventory inventory + first brewery-list
                set last-shipment first brewery-list
                set bank-account (bank-account - first brewery-list * brewer-purchase-cost)
                set brewery-list butfirst brewery-list
            ]
        ]
        if current-day day = "Tuesday"
        [
            supply-downstream
        ]
        if current-day day = "Friday"
        [
            place-order next-order
        ]
    ]
end

to-report role-coefficient [ role ]
    if role = "brewer"
    [
        report 1
    ]
    if role = "wholesaler"
    [
        report 2
    ]
    if role = "retailer"
    [
        report 4
    ]
end


to supply-downstream
    let this-week 0
    ;; pop the first guy off
    without-interruption
    [
        set this-week first waiting-list
        set waiting-list lput [] butfirst waiting-list
    ]
    let keep-looping true
    ;; go through the shit from this week
    while [keep-looping = true]
    [
        if inventory = 0
        [
            set keep-looping  false
        ]
        ifelse everybody-filled? this-week
        [
            set keep-looping  false
        ]
        [
            set this-week fill-first-order this-week
        ]
    ]

    if inventory = 0 ; push stuff into being due next week
    [
        set waiting-list replace-item 0 waiting-list merge-weeks (reverse this-week) first waiting-list
    ]
end

to-report merge-weeks [left-week right-week] ;;returns the result of merging the two weeks passed as params. I guess netlogo doesn't have some sort of list merging operator...
    ifelse length left-week = 0
    [
        report right-week
    ]
    [
        report merge-weeks butfirst left-week fput first left-week right-week
    ]
end

to-report fill-first-order [ open-week ] ;;allocates one unit from inventory to the first open order, removes empty orders from this weeks queue
    let new-order []

    if inventory < 1
    [
        report open-week
    ]

    if length open-week = 0
    [
        report []
    ]

    if not is-number? (first butfirst first open-week)
    [
        output-show "openWeek: " + open-week
        output-show 3 / 0
    ]

    ifelse ((first butfirst first open-week) < 1) ;; in other words, this order has been filled
    [
        report fill-first-order butfirst open-week
    ]
    [
        let to-whom first first open-week
        ;;increase the recipient's inventory by one
        without-interruption
        [
            ask players with [user-id = to-whom]
            [
                set inventory inventory + 1
                set bank-account (bank-account - role-coefficient player-type * brewer-purchase-cost) ;charge the client for the one beer
                set last-shipment last-shipment + 1
            ]
            set inventory inventory - 1
            set bank-account (bank-account + role-coefficient player-type * brewer-sale-price) ;add money to my account
            let new-amount ( ( first butfirst first open-week) - 1 )
            set new-order list to-whom new-amount
        ]
        report lput new-order butfirst open-week
    ]
end

to-report sum-of-orders-for-week [aWeek] ;; report the sum of the orders that are due this week
   ifelse (length aWeek) = 0
    [
        report 0
    ]
    [ report first butfirst first aWeek + sum-of-orders-for-week butfirst aWeek ]
end

to-report everybody-filled? [ orders ] ;; report true if all the orders are down to zero
    ifelse 0 = length orders
    [
        report true
    ]
    [
        ifelse 0 = first butfirst first orders
        [
            report everybody-filled? butfirst orders
        ]
        [
            report false
        ]
    ]
end

to initialize-player [ new-user-id new-player-type new-supplier new-inventory ]
  output-show new-user-id + " has supplier " +  new-supplier + " inv = " + new-inventory
  set user-id new-user-id
  set clients []
  set supplier new-supplier
  set inventory new-inventory
  set player-type new-player-type
  reset-player-values
  if supplier != nobody
  [  ask supplier
     [ add-new-client myself ]
  ]
end

to reset-player-values
  output-show "reset player values for " + user-id
  ifelse (player-type = "retailer")
  [
    set last-week-demand consumer-demand * 7 ; ; not used
    set waiting-list [ [] ]
    set inventory starting-retailer-inventory
  ]
  [
    ifelse (player-type = "wholesaler")
    [
        set waiting-list [ [] [] [] [] ]
        set last-week-demand floor (consumer-demand * 7 * (ratio-retailer / ratio-wholesaler))
        set inventory starting-wholesaler-inventory
    ]
    [ ;; brewer
        set waiting-list [ [] [] [] [] ]
        set last-week-demand floor (consumer-demand * 7 * (ratio-retailer / ratio-brewer ) )
        set brewery-list (list last-week-demand last-week-demand )
        set inventory starting-brewer-inventory
    ]
  ]
  set last-shipment last-week-demand
  set next-order last-shipment
  set bank-account 100
  set revenue-not-gotten 0.00
end

to process-message [ incoming-message ]
    if incoming-message = "^"
    [
        set next-order next-order + 7
    ]
    if incoming-message = "v" and next-order > 0
    [
        set next-order next-order - 7
    ]
    hubnet-send user-id "Next Order To Place" next-order
    ;    sendInfoToClient
end

to send-one-time-info
  hubnet-send user-id "You are a" player-type
  ifelse player-type = "brewer"
  [ hubnet-send user-id "Your Supplier Is" "not applicable" ]
  [ hubnet-send user-id "Your Supplier Is" user-id-of supplier ]
  hubnet-send user-id "Sale Price per Beer" "$" + role-coefficient player-type * brewer-sale-price + "0"
  ifelse player-type = "brewer" ;to prevent purchase price being $0.250
  [ hubnet-send user-id "Purchase Cost per Beer" "$" + role-coefficient player-type * brewer-purchase-cost ]
  [ hubnet-send user-id "Purchase Cost per Beer" "$" + role-coefficient player-type * brewer-purchase-cost + "0" ]
  hubnet-send user-id "Storage Cost per Beer" "$" + role-coefficient player-type * brewer-storage-cost + "0"
end

to send-info-to-client
  hubnet-send user-id "Inventory On Hand" inventory
  hubnet-send user-id "Last Incoming Shipment" last-shipment
  ;hubnet-send user-id "Waiting List" waiting-list
  hubnet-send user-id "Today Is" represent-day day
  hubnet-send user-id "Next Order To Place" next-order
  ifelse player-type = "retailer"
  [
      hubnet-send user-id "Current Downstream Demand" "" + (consumer-demand * 7) + " per week"
  ]
  [
      hubnet-send user-id "Current Downstream Demand" last-week-demand
  ]
  ifelse player-type = "retailer"
  [
      hubnet-send user-id "Potential Revenue Missed" "$" + precision revenue-not-gotten 2 + "0"

      hubnet-send user-id "Requested for delivery this week" "not applicable"
  ]
  [
      hubnet-send user-id "Potential Revenue Missed" "not applicable"
      hubnet-send user-id "Requested for delivery this week" sum-of-orders-for-week first waiting-list
  ]
  ifelse day < 2 ;if it's before the first Tuesday, don't show the extra 0
  [
      hubnet-send user-id "Bank Account" "$" + precision bank-account 2
  ]
  [
      hubnet-send user-id "Bank Account" "$" + precision bank-account 2 + "0"
  ]
end

to build-all-initial-waiting-lists
    ask players with [(player-type = "brewer") or (player-type = "wholesaler")]
      [ build-my-initial-waiting-list ]
end

to build-my-initial-waiting-list ; for when we start the game, to be called after everyone has joined
    output-show "building " + user-id + "'s initial waiting list using: " + waiting-list
    let do-times length waiting-list
    set waiting-list []
    repeat do-times
     [ set waiting-list fput make-one-week-initial-waiting-list waiting-list ]
end

to-report make-one-week-initial-waiting-list ; player procedure
    let my-output []
    let incredible-shrinking-clients clients
    repeat length clients
    [
      set my-output fput  (list user-id-of first incredible-shrinking-clients next-order-of first incredible-shrinking-clients )  my-output
      set incredible-shrinking-clients butfirst incredible-shrinking-clients
    ]
    report my-output
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Communicate with Clients ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to listen-clients
  while [ hubnet-message-waiting? ]
  [
    hubnet-fetch-message
    if not ( hubnet-enter-message? or hubnet-exit-message? )
    [
      ask players with [ user-id = hubnet-message-source ]
        [ process-message hubnet-message-tag ]
    ]
  ]
end

to initial-login
  while [ hubnet-message-waiting? ]
  [
    hubnet-fetch-message
    ifelse hubnet-enter-message?
    [ on-user-connect hubnet-message-source ]
    [
      if hubnet-exit-message?
      [
        remove-user hubnet-message-source
      ]
    ]
  ]
end

to on-user-connect [ new-user-id ]
  set user-numbers-to-names lput new-user-id user-numbers-to-names
  create-custom-players 1
  [
    ht
    initialize-player new-user-id ( get-role get-user-number new-user-id )
                                  ( get-best-supplier get-role get-user-number new-user-id )
    default-start-inventory get-role get-user-number new-user-id
    send-info-to-client
    send-one-time-info
  ]
end

to remove-user [ id ]
 set user-numbers-to-names remove id user-numbers-to-names
 ask players with [ user-id = id ] [ remove-client self die ]
end

to-report default-start-inventory [ role ]
  ifelse role = "retailer"
  [ report starting-retailer-inventory ]
  [
     ifelse role = "wholesaler"
     [ report starting-wholesaler-inventory ]
     [ report starting-brewer-inventory ]
  ]
end

to-report get-best-supplier [ role ]
  ifelse role = "retailer"
  [  report min-one-of (players with [player-type = "wholesaler"]) [ length clients ] ]
  [
    ifelse role = "wholesaler"
    [ report min-one-of (players with [player-type = "brewer"]) [ length clients ] ]
    [ report nobody ]
  ]
end

to-report get-user-number [ id ]
  report position id user-numbers-to-names
end

to-report get-user-name [ user-number ]
  report item user-number user-numbers-to-names
end

to-report get-role [ n ] ;; get the role of the nTh person to join
                       ;; outputs "brewer", "wholesaler", or "retailer" depending upon what the new user should be
    let post-mod n mod ( ratio-brewer + ratio-wholesaler + ratio-retailer)
    ifelse post-mod < ratio-brewer
    [ report "brewer" ]
    [
        ifelse post-mod < ratio-wholesaler + ratio-brewer
        [ report "wholesaler" ]
        [ report "retailer" ]
    ]
end


; *** NetLogo Model Copyright Notice ***
;
; This activity and associated models and materials was created as part of the projects:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN CLASSROOMS and
; INTEGRATED SIMULATION AND MODELING ENVIRONMENT.
; These projects gratefully acknowledge the support of the
; National Science Foundation (REPP & ROLE programs) -- grant numbers
; REC #9814682 and REC-0126227.
;
; Copyright 2003 by Uri Wilensky & Walter Stroup. Updated 2003. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from the copyright holders.
; Contact the copyright holders for appropriate licenses for redistribution
; for profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. & Stroup, W. (2003).  NetLogo HubNet Beer Game Alternate 2 model.
; http://ccl.northwestern.edu/netlogo/models/HubNetBeerGameAlternate2
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2003 by Uri Wilensky and Walter Stroup.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/HubNetBeerGameAlternate2
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
505
21
725
58
17
0
6.0
1
10
1
1
1
0
0
0
1

CC-WINDOW
5
307
734
402
Command Center
0

BUTTON
76
145
215
178
go
go
T
1
T
OBSERVER
T
NIL

SLIDER
241
70
481
103
starting-brewer-inventory
starting-brewer-inventory
0
200
126
1
1
NIL

SLIDER
242
142
482
175
starting-retailer-inventory
starting-retailer-inventory
0
50
21
1
1
NIL

SLIDER
242
106
482
139
starting-wholesaler-inventory
starting-wholesaler-inventory
0
100
63
1
1
NIL

SLIDER
242
178
482
211
consumer-demand
consumer-demand
0
50
36
1
1
NIL

BUTTON
76
73
215
106
initial-login
initial-login
T
1
T
OBSERVER
T
NIL

MONITOR
282
19
441
68
Day
represent-day day
3
1

BUTTON
145
37
215
70
reset
reset
NIL
1
T
OBSERVER
T
NIL

BUTTON
76
109
215
142
ready to go
build-all-initial-waiting-lists
NIL
1
T
OBSERVER
T
NIL

OUTPUT
49
219
611
293

BUTTON
76
37
142
70
NIL
setup
NIL
1
T
OBSERVER
T
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This is a HubNet Participatory simulation of the Beer Game. The beer game is a model of a product distribution system.

Players play as either retailers (who see demand from consumers and order from wholesalers), wholesalers (who see demand from retailers and order from brewers), or brewers (who see demand from wholesalers and produce the beer to give to wholesalers). Players try to maximize their profit, which is derived from selling the most beer possible. Buying beer and keeping it in inventory costs money. Crucial is the fact that beer is not delivered immediately upon order placement; rather, retailers and wholesalers wait an average of four weeks to receive their orders. Wholesalers and brewers maintain a waiting list of orders received but not yet filled. Beer to satisfy this waiting list is alloted as soon as possible.

NOTE: "Beer Game Alternate 1" and "Beer Game Alternate 2" are supplied with this release as examples of other possible approaches to designing and implementing a Beer Game activity.  The main Beer Game activity is the one we have developed most fully in terms of: design; reliability and robustness; documentation; and quality of code.  However, we felt that users might also be interested in seeing these other implementations.  Be cautious if intending to use the alternates with an actual group.


HOW TO USE IT
-------------
For the administrator, press the INITIAL-LOGIN button and leave it pressed while participants log onto the simulation. ( there is no need to press the SETUP button unless you want to start over with new clients )   Once everyone is logged on, click the READY TO GO button.  When you are ready to begin play, click GO and leave it depressed until you wish to end play. If you want to start over with the same clients press RESET if you want different clients tell everyone to logout and press setup. STARTING-BREWER-INVENTORY, STARTING-WHOLESALER-INVENTORY, and STARTING-RETAILER-INVENTORY should be set before allowing clients to login.  You can change CONSUMER-DEMAND at any time during the activity.

For players, the only actions required while playing the game are increasing and decreasing the size of orders placed each week. The interface tells players what their role is, who their supplier is, and the financial costs and revenues per unit of beer. Orders are sent to the supplier using whatever value is in the Next Order to Place monitor on Friday.  Orders are received on Tuesday.  Inventory costs are assessed on Tuesday also.

The REQUESTED FOR DELIVERY monitor (only used by wholesalers and brewers) tells users how much beer should be delivered in the current week. For example, if two units were ordered four weeks ago to be delivered this week and two units last week went unfilled, the requested for delivery monitor would indicate four.


THINGS TO NOTICE
----------------
While playing, players should try to make note of how demand changes over time. This can be useful in analyzing the behavior of the system at the end of the game.


THINGS TO TRY
-------------
The administrator of the game should initially let consumer demand remain stable. Once players are comfortable with the steady state of the system, the administrator should double or triple consumer demand. Players who are unfamiliar with the emergent dynamics of the system will often greatly exaggerate the change in demand when placing their own orders. Usually, this results in bankrupcy.


EXTENDING THE MODEL
-------------------
Users could modify the model to change the latencies of the game or to adjust the ratio of retailers to wholesalers to brewers.  Also, try to add an additional role in the system, such as a distributor.  See if this changes the dynamics of the system.


NETLOGO FEATURES
----------------
This model uses NetLogo's HubNet to handle interactions with players. A future version of this model will use NetLogo's nascent  peer-to-peer networking to allow players to customize the information they see during game play, as well as write code to play on their behalf.


RELATED MODELS
--------------
Tragedy of the Commons


CREDITS AND REFERENCES
----------------------
See Senge's "The Fifth Discipline" for more information about the beer game.

This activity and associated models and materials was created as part of the projects:
PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN CLASSROOMS and INTEGRATED SIMULATION AND MODELING ENVIRONMENT. These projects gratefully acknowledge the support of the National Science Foundation (REPP & ROLE programs) -- grant numbers REC #9814682 and REC-0126227.

Copyright 2003 by Uri Wilensky & Walter Stroup. All rights reserved.

Permission to use, modify or redistribute this model is hereby granted, provided that both of the following requirements are followed:
a) this copyright notice is included.
b) this model will not be redistributed for profit without permission from the copyright holders.
Contact the copyright holders for appropriate licenses for redistribution for profit.

To refer to this model in academic publications, please use: Wilensky, U. & Stroup, W. (2003). NetLogo HubNet Beer Game Alternate 2 model. http://ccl.northwestern.edu/netlogo/models/HubNetBeerGameAlternate2. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2003 by Uri Wilensky and Walter Stroup.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/HubNetBeerGameAlternate2 for terms of use.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0beta6
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
