globals [
  ;;holds patches that generate the dice pairs, singles
  generator-patches

  ;;offset of patches -- the model uses the 'offset-pxcor'
  ;;variable of patches instead of the pxcor
  pxcor-offset

  single-die-outcomes-list
  dice-pair-outcomes-list

  #runs

  ;;hold the color values for the columns (used in setup)
  column1-color
  column2-color
  mid-column-color
]

breeds [
  pairs
  singles
  clients
]

patches-own [
  sum-to-match   ;; grabs pairs
  num-to-match   ;; grabs singles
  offset-pxcor
]

pairs-own [
  sum-of-roll    ;;the sum of this die's value and it's partner's value
]

;;; pairs and singles both own these
turtles-own [
  die-value
  moving?   ;;false if already added to the display histogram
]

clients-own
[
  user-id
  ;;hold the values of the dice that are put in by the user
  die1
  die2
]

;;;;;;;;;;;;;;;;;;;;;;
;; Setup Procedures ;;
;;;;;;;;;;;;;;;;;;;;;;

to startup
  setup
  hubnet-set-client-interface "COMPUTER" [ "clients/Dice Stalagmite client.nlogo" ]
  hubnet-reset
end

to setup
  ca
  set column1-color 5
  set column2-color 34 ;; blue + 2 ;; old value = 35
  set mid-column-color 8
  set pxcor-offset -8
  clean-setup
end

to clean-setup
  ask singles [ die ]
  ask pairs [ die ]
  clear-all-plots
  set #runs 0

  ;;sets the offset xcor value for patches
  ask patches [
    set offset-pxcor pxcor - pxcor-offset
  ]

  ;;color right part of display, set the sum-to-match
  ask patches with [ offset-pxcor > 4 ] [
    set sum-to-match ((offset-pxcor - 1) / 2) - .5
    ifelse ceiling sum-to-match mod 2 = 0 [ set pcolor column1-color ] [ set pcolor column2-color ]
  ]

  ;;color left part of display, set the sum-to-match
  ask patches with [ offset-pxcor < -4 ] [
    set num-to-match screen-edge-x + pxcor + 1
    ifelse num-to-match mod 2 = 0 [ set pcolor column1-color ] [ set pcolor column2-color ]
  ]

  ;;sets the generator patch values
  set generator-patches patches with [
    pycor = screen-edge-y and (
      offset-pxcor = -1 or
      offset-pxcor = 0
    )
  ]

  ;;color middle columns
  ask patches with [ abs offset-pxcor < 5 ] [
    set pcolor mid-column-color
  ]

  set single-die-outcomes-list []
  set dice-pair-outcomes-list []
end

;;;;;;;;;;;;;;;;;;;;;;;;
;; Runtime Procedures ;;
;;;;;;;;;;;;;;;;;;;;;;;;

to go
  ;;bumps down the doubles display histogram when there is a dice-pair
  ;;at the very top of the screen and the stop-at-top? is set to false
  while [ not stop-at-top? and any? pairs with [ moving? = false and  (pycor = screen-edge-y) and (offset-pxcor > 4) ] ]
  [
     bump-down-pairs
  ]

  ;;bumps down the singles display histogram when there is a die at the very top of the screen
  while [ any? singles with [ moving? = false and (pycor = screen-edge-y - 2) and (offset-pxcor < -4)] ]
  [
     bump-down-singles
  ]

  ;;if there is a die that is not yet static
  ;;in the histogram, updates its position
  if any? turtles with [ moving? = true ]
  [
    go-pairs
    go-singles
  ]

  check-hubnet-messages
end

to check-hubnet-messages
  if hubnet-message-waiting?
  [
    hubnet-fetch-message
    ;;creates a new client turtle if new HubNet client joining
    ifelse hubnet-enter-message?
    [
      if not any? clients with [ user-id = hubnet-message-source ]
      [
        ;;creates the actual client, sets the values of dice to 0
        ;;and sets the user-id variable to the hubnet-source
        cct-clients 1
        [
          ht
          set user-id hubnet-message-source
          set die1 0
          set die2 0
        ]
      ]
    ]
    [
      ;;if a HubNet client quit, kill client turtle
      ifelse hubnet-exit-message?
      [
         ask clients with [ user-id = hubnet-message-source ]  [ die ]
      ]
      ;;if just a message from the client, execute appropriately
      [ if any? clients [ exe-cmd ] ]
    ]
  ]
end

to exe-cmd
  let this-user hubnet-message-source
  let value hubnet-message
  if value = "--" [ set value 0 ]
  ;;if the change was a change in the first die, change the die value of that client
  if hubnet-message-tag = "Die_A"
  [
    set die1-of one-of clients with [ user-id = this-user ] value
  ]
  ;;if the change was a change in the second die, change the die value of that client
  if hubnet-message-tag = "Die_B"
  [
    set die2-of one-of clients with [ user-id = this-user ] value
  ]
  ;;if client hit SUMBMIT, check so that the die values of the client are not 0
  if hubnet-message-tag = "Submit"
  [
    ifelse any? clients with [ user-id = this-user ]
       and (die1-of one-of clients with [ user-id = this-user ] = 0 or
       die2-of one-of clients with [ user-id = this-user ] = 0)
    [
      ;;send message
      hubnet-send this-user "Message:" "Please select values for both dice."
    ]
    [
      ;;clears the user's interface
      hubnet-send this-user "Die_A" "--"
      hubnet-send this-user "Die_B" "--"

      ;;finds the height of the tallest pair column
      let height-of-tallest-column 0
      if dice-pair-outcomes-list != []
      [
        let mode-of-pairs-list first modes dice-pair-outcomes-list
        set height-of-tallest-column length filter [ ? = mode-of-pairs-list ] dice-pair-outcomes-list
      ]

      ;;if stop-at-top? is false and the pair side of the bar-chart is not full,
      ;;generate the pair and the singles
      ifelse not (stop-at-top? and height-of-tallest-column >= screen-size-y)
      [
        without-interruption
        [
          generate-roll die1-of one-of clients with [ user-id = this-user ]
                        die2-of one-of clients with [ user-id = this-user ]
        ]
        ;;send confirmation message to user
        hubnet-send this-user "Message:" "Thank you. Your input was " +
              die1-of one-of clients with [ user-id = this-user ] + "-" +
              die2-of one-of clients with [ user-id = this-user ] + "."
      ]
      [
        ;;if stop-at-top? is true and the pair side of the bar-chart is full,
        ;;stops the simulation -- sends an appropriate message ot user
        hubnet-send this-user "Message:" "GAME OVER!"
      ]
      ;;resets the client-turtle's variables
      set die1-of one-of clients with [ user-id = this-user ] 0
      set die2-of one-of clients with [ user-id = this-user ] 0
    ]
  ]
end

to go-pairs
  let moving-pairs pairs with [ moving? = true ]
  ;;if a pair is not at the correct column, move on
  ask moving-pairs with [ sum-of-roll != sum-to-match and heading = 90]
  [
    fd 1
  ]
  ;;if moving down, check if at the bottom, them move on
  ask moving-pairs with [ sum-of-roll = sum-to-match ]
  [
    set heading 180

    ;;if at the bottom of the screen, check it the pair should
    ;;go "under water"
    if pycor = (- screen-edge-y) [ should-I-die?-pairs ]

    ;;if reaches the bottom (and the pair is "above water") or there is a pair in front, stops
    ifelse pycor = (- screen-edge-y ) or
           any? value-from patch-ahead 1 [ pairs-here with [ moving? = false ] ]
    [
      set moving? false
      ;;updates the list, plots
      if ceiling sum-to-match = sum-to-match
      [
        set dice-pair-outcomes-list (lput sum-to-match dice-pair-outcomes-list)
        update-plots
      ]
    ]
    ;;if not at bottom, move on down
    [
      fd 1
    ]
  ]
end

;;;moves the singles a step either to the correct column, down the column or stops the single
to go-singles
  let moving-singles singles with [ moving? ]
  ;;if a single is not at the correct column, move on
  ask moving-singles [
    if die-value = num-to-match [
      ;;move down
      set heading 180
    ]

    ;;if at the bottom of the screen, check it the single should
    ;;go "under water"
    if pycor = (- screen-edge-y) [ should-I-die?-singles ]

    ;;if reaches the bottom (and the pair is "above water") or there is a single in front, stops
    ifelse pycor = (- screen-edge-y ) or
           any? value-from patch-ahead 1 [ singles-here with [ not moving? ] ]
    [
      ;;updates the list, plots
      set moving? false
      set single-die-outcomes-list (lput die-value single-die-outcomes-list)
      update-plots
    ]
    [
      ;;if nothing ahead of the single and it is not at the bottom, it moves one down the column
      fd 1
    ]
  ]
end

;;creates a new pair of dice (both singles and pairs) [given the die values]
to generate-roll [ die-1 die-2 ]
  let the-values (list die-1 die-2)
  let t 0
  ;;ask each generator patch to create a pair (one die, with breed pair), with the values
  ask generator-patches [
    sprout-pairs 1 [
      set color white
      set die-value item t the-values
      set shape "die " + die-value
      set moving? true
      set heading 90
      if t = 0 [ set sum-of-roll -.5 ]
      set t (t + 1)
      set sum-of-roll sum-of-roll + sum the-values
    ]
  ]

  set t 0
  ;;clones the pairs to make to the singles with the pair's value
  ask generator-patches [
    sprout-singles 1 [
      set color white
      set die-value item t the-values
      set shape "die " + die-value
      set moving? true
      set heading 270
      set t (t + 1)
    ]
  ]

  ;;aggregates the number of runs
  set #runs (#runs + 1)
end

;;moves each single ahead one, kills ones that are on bottom of screen
to bump-down-singles
  without-interruption
  [
    ask singles with [ offset-pxcor < -4 and moving? = false ]
    [
      ifelse ycor = (- screen-edge-y)
        [
          die
        ] [
          fd 1
        ]
    ]
  ]
end

;;moves each pair ahead one, kills ones that are on bottom of screen
to bump-down-pairs
  without-interruption
  [
    ask pairs with [ offset-pxcor > 4  and moving? = false ]
    [
      ifelse ycor = (- screen-edge-y)
        [
          die
        ] [
          fd 1
        ]
      ]
  ]
end

to should-I-die?-pairs  ;;turtle procedure
  if dice-pair-outcomes-list != []
  [
    ;; determines if the dice-pair's column is tall enough to be seen in the truncated view
    let mode-of-pairs-list first modes dice-pair-outcomes-list
    let height-of-tallest-column length filter [ ? = mode-of-pairs-list ] dice-pair-outcomes-list
    let height-of-my-column length filter [ ? = ceiling sum-of-roll ] dice-pair-outcomes-list
    let my-diff ( height-of-tallest-column - height-of-my-column )

    if my-diff >= screen-size-y - 1 + 1
    [
      ;;updates the list, plots
      if ceiling sum-to-match = sum-to-match
      [
        set dice-pair-outcomes-list (lput sum-to-match dice-pair-outcomes-list)
        update-plots
      ]
      die
    ]
  ]
end

to should-I-die?-singles  ;;turtle procedure
  if single-die-outcomes-list != []
  [
    ;; determines if the dice-pair's column is tall enough to be seen in the truncated view
    let mode-of-dice-list first modes single-die-outcomes-list
    let height-of-tallest-column length filter [ ? = mode-of-dice-list ] single-die-outcomes-list
    let height-of-my-column length filter [ ? = die-value ] single-die-outcomes-list
    let my-diff ( height-of-tallest-column - height-of-my-column )

    if my-diff >= screen-size-y - 1 - 2 + 1
    [
      ;;updates the list, plots
      set single-die-outcomes-list (lput die-value single-die-outcomes-list)
      update-plots
      die
    ]
  ]
end

to random-roll
  ;;generates a random pair
  generate-roll (1 + random 6) (1 + random 6)
end

;;runs the simulation num times
to simulate [ num ]
  let iter 0
  loop
  [
    if iter >= num or (stop-at-top? and
       any? pairs with [ moving? = false and  (pycor = screen-edge-y) and (offset-pxcor > 4) ])
    [ repeat 70 [ go ]  stop ]
    random-roll go go
    set iter iter + 1
  ]
end

;;updates the histograms
to update-plots
    set-current-plot "Single Die"
    histogram-list single-die-outcomes-list
    set-current-plot "Dice Pairs"
    histogram-list dice-pair-outcomes-list
end

; *** NetLogo Model Copyright Notice ***
;
; This activity and associated models and materials was created as part of the projects:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN CLASSROOMS and 
; INTEGRATED SIMULATION AND MODELING ENVIRONMENT.
; These projects gratefully acknowledge the support of the 
; National Science Foundation (REPP & ROLE programs) -- grant numbers 
; REC #9814682 and REC-0126227.
;
; Copyright 2005 by Uri Wilensky. Updated 2005. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from the copyright holders.
; Contact the copyright holders for appropriate licenses for redistribution 
; for profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2005).  NetLogo HubNet Dice Stalagmite model.
; http://ccl.northwestern.edu/netlogo/models/HubNetDiceStalagmite.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2005 Uri Wilensky.  All rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/HubNetDiceStalagmite
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
177
43
557
584
18
25
10.0
1
10
1
1
1
0
0
0
1

CC-WINDOW
5
598
793
693
Command Center
0

BUTTON
7
5
62
38
Setup
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
63
5
118
38
Go
go
T
1
T
OBSERVER
T
NIL

PLOT
12
43
172
584
Single Die
Die Value
Count
1.0
7.0
0.0
51.0
true
false
PENS
"default" 1.0 1 -16777216 true

PLOT
563
43
784
584
Dice Pairs
Dice Total
Count
2.0
13.0
0.0
51.0
true
false
PENS
"default" 1.0 1 -16777216 true

MONITOR
249
85
327
134
Total Pairs
#runs
0
1

SWITCH
389
5
509
38
stop-at-top?
stop-at-top?
1
1
-1000

BUTTON
119
5
174
38
Clean
clean-setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
232
5
341
38
Random Roll
if not any? generator-patches with [ any? singles-here ]\n[ random-roll ]
NIL
1
T
OBSERVER
T
NIL

@#$#@#$#@
WHAT IS IT?
-----------
Dice Stalagmite PSA is a Participatory Simulation Activity (PSA) in probability for exploring dependent and independent events.  Specifically, you compare the outcome distribution of a compound event, the sum of two randomly "rolled" dice, to the outcome distribution of independent events, the values of these same dice, taken one die at a time.

This model is a part of the ProbLab curriculum.  The ProbLab curriculum is currently under development at the CCL.  For more information about the ProbLab Curriculum please refer to http://ccl.northwestern.edu/curriculum/ProbLab/.


HOW IT WORKS
------------
Participants each roll their own pair of dice (preferably dice of different colors) and input the outcome, for example, "5 2", through their client interface.  Each outcome appears at the top middle of the server interface, in the form of the same two dice.  Then, this pair is duplicated, and one pair moves to the right (the pair), and the other pair moves to the left (the singles).  On each side, there is a "picture bar chart": on the right, the bar chart is for sums of dice (2 through 12) and on the left, the bar chart is for individual die values (1 through 6).  The dice have to find their columns and slide down those columns.  On the right, the dice find the column of their sum, and on the left, the dice each finds its own column of its value.  The events in each of the charts are mirrored in their respective histograms.

As the dice stack up, you begin to notice different typical distributions in each bar chart.  The "singles" chart (SINGLE DIE), on the left, grows to become equally distributed.  It's never really "flat," but we can never predict in advance which column will "win."  On the other hand, the "pairs" bar chart (DICE PAIR TOTAL), on the right, always grows to the shape of a triangle -- the closer a column is to the middle of the of chart, the taller it will be, eventually.


HOW TO USE IT
-------------
Make sure all participants are connected and have their clients.  Press GO.  Participants each roll their own pair of dice and then use their interface to input the result of the roll.  It is helpful (but not absolutely important) to have dice of different colors, because then you can differentiate between, say, a "2 5" and a "5 2."  That is helpful for comparing the outcomes to the combinatorial analysis of a pair of dice.

Make sure your participants understand the difference between the two charts and how the dice find their columns (independently, on the left, and as a pair, on the right).  You can use the RANDOM ROLL and set a low speed in order to do that.

Once you see that the sums chart is beginning to grow its "bump" in the middle, ask the participants if they are noticing anything.  If not, keep playing until participants notice the bump.  If you have worked on the combinatorial analysis of a pair of dice, participants may realize that they are getting in the pairs chart (DICE PAIR TOTAL) the same triangular shape they got in that analysis (see the ProbLab materials for this triangular shaped combinatorial space of a pair of dice).  A useful discussion could be around why an experiment with random outcomes mirrors the shape of a chart you built through careful analysis.  Note that the combinatorial-analysis chart has no duplicates -- each combination appears only once -- whereas the experiment does have duplicates.

Because the single-die chart fills up faster than the dice-pair chart, the program bumps the single-die columns down, so that they can keep growing.  You can set the STOP-AT-TOP? switch to "Off" to do the same for the DICE PAIR TOTAL chart.  The plots, however, keep a record of all the events.

Buttons:
SETUP -- initializes variables, clears the plots, disables currently-open clients and draws the display.

GO -- enables participants to input their data.

CLEAN -- initializes variable and clears the plots.

RANDOM ROLL -- creates a random pair of dice that behaves the same as though it had been input by a client.  This is useful for preparing for class, for demonstrations to the classroom, and for boosting the activity of a relatively small group of participants.

Switches:
STOP-AT-TOP? -- when set to "Off," the stacks of dice in the dice-pair bar chart all bump down one row each time one of the stacks reaches the top of the chart (this happens automatically for the singles bar chart, but it's an option with the dice-pair bar chart).  Otherwise, the simulation will stop when the dice-pair bar chart in the display is filled to the top.

Monitors:
TOTAL PAIRS -- keeps a record of how many pairs have been processed.

Plots:
SINGLE DIE -- histogram of independent outcomes of rolling dice (each die value is a separate event).

DICE PAIR TOTAL -- histogram of compound outcomes from rolling dice (each sum of a pair of dice is an event).


THINGS TO NOTICE
----------------
The more events you stack up in the charts, the more the DICE PAIRS TOTAL chart will take on a triangular shape.  Eventually, the '7' column will be the tallest, and then, in descending order, the '5' and '8,' the '4' and '9,' the '3' and '10,' the '2' and '11,' and the '1' and '12.'  The chart will not necessarily be perfect, but the more input the program gets, the closer the chart will approach this shape.

At the same time, the SINGLE DIE chart approaches the shape of an equal distribution.
The SINGLE DIE is never exactly equally distributed.  If you look at the chart, you'll see that one of the columns is "beating" the other columns.  So why do we call it "equally distributed" if it's not really equal?  Well, it may depend on whether we are attending to the differences between the heights of the columns (an "additive" approach) or if we're attending to the proportions of the heights (a "multiplicative" approach).  Once you've accumulated hundreds of pairs of dice, one column may be over ten rows taller than another, but when you look at its plot, SINGLE DIE, this difference will not appear as large as it is in the bar chart of the display.


THINGS TO TRY
-------------
If you are working alone on this PSA, for instance, if you are preparing for class, press GO and then type in the Command Center: simulate 1000.  This will allow you to see how the charts will look after, say, 30 students have each input 34 rolls.


EXTENDING THE MODEL
-------------------
Why settle for a pair of dice?  How would the charts look for three dice?  Do you expect any difference in the shapes of the outcome distributions?  Would the SINGLE-DIE chart change at all?


NETLOGO FEATURES
----------------
Sometimes the model developer finds it useful to change the coordinate system of the display for convenience.  In this model, we made a new variable for patches that held its "offset x-value."  In fact, the model does not use the "pxcor" variable of the patches at all.  In effect, the whole model shifts.  This is convenient, because we put the 0 of the model at a point between the two displays, so when the model refers to the points on the screen, it does so in general terms of that new 0, instead of the display's default "pxcor" 0.  This technique also allows for easier extension of the model.


RELATED MODELS
--------------
See the ProbLab model Dice Stalagmite that is essentially the same activity but is designed to be used by a single person.  See the ProbLab models 9-Block Stalagmite and Sample Stalagmite PSA that convey the same ideas as this model does but use a different stochastic object (not a pair of dice).


CREDITS AND REFERENCES
----------------------
Thanks to Dor Abrahamson for his work on the design of this model and the ProbLab curriculum.  Thanks to Josh Unterman for building Dice Stalagmite and to Steve Gorodetskiy for converting it to the Dice Stalagmite PSA.

This activity and associated models and materials was created as part of the projects: PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN CLASSROOMS and INTEGRATED SIMULATION AND MODELING ENVIRONMENT. These projects gratefully acknowledge the support of the National Science Foundation (REPP & ROLE programs) -- grant numbers REC #9814682 and REC-0126227.

Copyright 2005 by Uri Wilensky. Updated 2005. All rights reserved.

Permission to use, modify or redistribute this model is hereby granted, provided that both of the following requirements are followed:
a) this copyright notice is included.
b) this model will not be redistributed for profit without permission from the copyright holders.
Contact the copyright holders for appropriate licenses for redistribution for profit.

To refer to this model in academic publications, please use: Wilensky, U. (2005). NetLogo HubNet Dice Stalagmite model. http://ccl.northwestern.edu/netlogo/models/HubNetDiceStalagmite. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2005 Uri Wilensky. All rights reserved. See http://ccl.northwestern.edu/netlogo/models/HubNetDiceStalagmite for terms of use. 
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 30 30 240

circle 2
false
0
Circle -7500403 true true 16 16 270
Circle -16777216 true false 46 46 210

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

die 1
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 129 129 42

die 2
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 69 69 42
Circle -16777216 true false 189 189 42

die 3
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 69 69 42
Circle -16777216 true false 129 129 42
Circle -16777216 true false 189 189 42

die 4
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 69 69 42
Circle -16777216 true false 69 189 42
Circle -16777216 true false 189 69 42
Circle -16777216 true false 189 189 42

die 5
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 69 69 42
Circle -16777216 true false 129 129 42
Circle -16777216 true false 69 189 42
Circle -16777216 true false 189 69 42
Circle -16777216 true false 189 189 42

die 6
false
0
Rectangle -7500403 true true 45 45 255 255
Circle -16777216 true false 84 69 42
Circle -16777216 true false 84 129 42
Circle -16777216 true false 84 189 42
Circle -16777216 true false 174 69 42
Circle -16777216 true false 174 129 42
Circle -16777216 true false 174 189 42

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0beta6
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
