;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Variable and Breed declarations ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

globals
[
  ;; quick start instructions variables
  quick-start  ;; current quickstart instruction displayed in the quickstart monitor
  qs-item      ;; index of the current quickstart instruction
  qs-items     ;; list of quickstart instructions

  ;; lists used to create the various turtles
  shape-names       ;; list that holds the names of the non-sick shapes a student's turtle can have
  carnivore-shape-names ;; list of possible shapes for student carnivores
  herbivore-shape-names ;; list of possible shapes for student herbivores

  colors            ;; list that holds the colors used for students' turtles
  color-names       ;; list that holds the names of the colors used for students' turtles
  used-shape-colors ;; list that holds the shape-color pairs that are already being used

  ;; misc
  clock                ;; keeps track of the number of times through the go procedure (if there is at least one turtle infected)
  max-possible-codes   ;; total number of possible unique shape/color combinations
  max-possible-herbivore-codes ;;code system is specialized for different species, as herbivores and carnivores have different shapes
  max-possible-carnivore-codes

  plant-count          ;; how many plants are currently on the screen
  total-bites          ;; totals the number of points that all herbivores have lost from getting bit
  total-meat-starve    ;; totals the number of points that carnivores have lost from starvation
  total-veggie-starve  ;; totals the number of points that herbivores have lost from starvation

  initial-student-herbivores ;; when students are initially created, the first few should be herbivores, to prevent initial starvation
  start-points         ;; the number of points students start with
]

turtles-own
[
  carnivore?    ;; if a turtle is an herbivore or carnivore
  points        ;; total number of points a turtle has, based on eating, starving, pacts, and bites
  base-shape   ;; original shape of a turtle
  step-size    ;; the amount that a turtle will go forward in the current direction
]

patches-own
[
  food      ;; the amount of food each patch currently contains
  max-food ;; the maximum amount of food on a patch
]

breeds
[
  androids  ;; created by the CREATE ANDROIDS button; not controlled by anyone, but can become sick and spread sickness
  students  ;; created and controlled by the clients
]

students-own
[
  user-id  ;; unique id, input by the client when they log in, to identify each student turtle
  old-color ;;when no longer starving, the student can go back to his/her original color
]


;;;;;;;;;;;;;;;;;;;;;
;; Setup Functions ;;
;;;;;;;;;;;;;;;;;;;;;

to startup
  setup
  setup-quick-start
  hubnet-set-client-interface "COMPUTER" [ "clients/Herbivore Carnivore Client.nlogo" ]
  hubnet-reset
  auto-plot-on
end

;; Initializes the display, and creates a list that contains the names of the shapes
;; used by turtles in this activity.  Also initializes the data lists.
to setup
  cp
  ct
  clear-output
  setup-vars
  setup-plot
  setup-patches
end

to rerun
  setup-plot
  setup-patches
  ask turtles [
    set heading (random 4) * 90
    fd random screen-size-x
    set points start-points
    set shape base-shape
  ]
end

;; initialize global variables
to setup-vars
  set clock 0

  set start-points 50
  set plant-count 0

  set initial-student-herbivores 0

  set shape-names [
                    "bear" "butterfly" "cat" "cow" "dog" "hawk"
                    "horse" "lobster" "monster" "moose" "rabbit"
                     "shark" "sheep" "squirrel" "turtle" "wolf"
                  ]

  set herbivore-shape-names [
                              "butterfly" "cow" "horse"
                              "moose" "rabbit" "sheep"
                              "squirrel" "turtle"
                            ]

  set carnivore-shape-names [ "bear" "cat""dog" "hawk" "lobster" "monster" "shark""wolf" ]

  set colors      [white   brown   yellow   green   lime   turquoise
                   cyan   sky   blue   violet ]
  ;; adjust a few colors so they don't blend in with the red infection dot too much
  set colors lput (orange + 1) colors
  set colors lput (magenta + 0.5) colors
  set colors lput (pink + 2.5) colors
  set color-names ["white" "brown" "yellow" "green" "lime" "turquoise"
                   "cyan" "sky" "blue" "violet" "orange" "magenta" "pink"]
  set max-possible-codes (length colors * length shape-names)
  set max-possible-herbivore-codes (length colors * length herbivore-shape-names)
  set max-possible-carnivore-codes (length colors * length carnivore-shape-names)
  set used-shape-colors []
end

to setup-patches
cp
 ask random-n-of 2 patches
    [
      set max-food 7000
    ]

  while [ max values-from patches [ max-food ] > 100 ]
     [ diffuse max-food 0.1 ]
  ask patches
  [
    set food round (max-food / 2)
    set pcolor scale-color (green + 2) food 0 100
  ]

end

;; initialize the plot

to setup-plot
  clear-all-plots
end

to do-plot
  set-current-plot "Total Points"  ;;this plots the total points of herbivores, and of carnivores
  set-current-plot-pen ("meat-avg-points")
  ifelse (any? turtles with [ carnivore? ] )
    [ plot mean values-from turtles with [ carnivore? ] [ points ] ]
    [ plot 0 ]

  set-current-plot-pen ("veggie-avg-points")
  ifelse (any? turtles with [ not carnivore? ] )
    [ plot mean values-from turtles with [ not carnivore? ] [ points ] ]
    [ plot 0 ]
end

;; creates turtles that wander at random
to create-android-herbivores
  create-custom-androids 1
  [
    setxy (random-float screen-size-x) (random-float screen-size-y)
    set color grey
    set heading (random 4) * 90
    set carnivore? false
    set points start-points
    set base-shape "sheep"
    set shape base-shape
    set step-size 1
    set label "H"
    set label-color blue
  ]
end

;; creates turtles that wander at random
to create-android-carnivores
  create-custom-androids 1
  [
    setxy (random-float screen-size-x) (random-float screen-size-y)
    set color grey
    set heading (random 4) * 90
    set carnivore? true
    set points start-points
    set base-shape "wolf"
    set shape base-shape
    set step-size 1
    set label "C"
    set label-color blue
  ]
end

to make-plants
  let quantity 0
  ask patches
  [
    if food < max-food
    [
      set quantity 3
      set food (food + quantity)
      set pcolor scale-color (green + 2) food 0 100
      set plant-count plant-count + quantity
    ]
  ]
end

;; give the user some information about what the setup button does so they can
;; know whether they want to proceed before actually doing the setup
to setup-prompt
  if user-yes-or-no? ("The SETUP button should only be used when starting "
              + "over with a new group (such as a new set of students) since "
              + "all data is lost.  Use the RE-RUN button for continuing with "
              + "an existing group."
              + "\n\nDo you really want to setup the model?")
  [
    user-message "Before closing this dialog, please do the following:"
                 + "\n  -Have everyone that is currently logged in, log off and "
                 + "then kick all remaining clients with the HubNet Console."
    setup
  ]
end


;;;;;;;;;;;;;;;;;;;;;;;
;; Runtime Functions ;;
;;;;;;;;;;;;;;;;;;;;;;;

to go
  ;; get commands and data from the clients
  listen-clients
  every 1 [ ask students [ update-points-monitor ] ]

  ;; allow the androids to wander around the view
  if wander?
    [ androids-wander ]

  plants-regrow
  check-animals
  do-plot
end

;; controls the motion of the androids
to androids-wander
  every android-delay
  [
    ask androids
    [
      ifelse (random 10 = 0)
        [ rt (90 * random 4) ]
      [ set heading uphill4 food ]
      fd move-amount
    ]
  ]
end

to plants-regrow
  let quantity 0
  every plant-regrowth-rate
  [
    ask patches with [ (food < max-food) ]
    [
        set quantity 3
        set food (food + quantity)
        set pcolor scale-color (green + 2) food 0 100
    ]
  ]
end

;; report the amount we can move in the current direction
;; if we don't want to wrap around the screen
to-report move-amount  ;; turtle procedure
  ;; the value of either pxcor or pycor depending upon if
  ;; we are moving vertically or horizontally
  let new-pcor 0
  ;; the value of either screen-edge-x or screen-edge-y
  ;; depending upon if we are moving vertically or horizontally
  let screen-edge 0

  ifelse heading mod 180 = 0
  [
    set screen-edge screen-edge-y
    ifelse heading = 0
    [ set new-pcor pycor + step-size ]
    [ set new-pcor pycor - step-size ]
  ]
  [
    set screen-edge screen-edge-x
    ifelse heading = 90
    [ set new-pcor pxcor + step-size ]
    [ set new-pcor pxcor - step-size ]
  ]
  ;; if we would step past a patch at the edge of the view,
  ;; make us step onto the edge patch
  if abs new-pcor > abs screen-edge
  [ report step-size - ( abs new-pcor - abs screen-edge ) ]
  report step-size
end

;; determines the gain and loss of points for each animal, based on eating, starving, and getting eaten.
to check-animals
  let eaten-turtle nobody
  ask turtles
  [
    set-starve-shape
    ifelse ( not carnivore? )
    [
      get-to-eat-veggie
     set points points * ( 100 - herbivore-starve-penalty ) / 100
    ]
    [
      set eaten-turtle ( random-one-of other-turtles-here with [ not carnivore? ] )
      if ( eaten-turtle != nobody ) and ( points-of eaten-turtle > start-points )
      [ get-to-eat-meat eaten-turtle ]
      set points points * (100 - carnivore-starve-penalty) / 100
    ]

   ]
end

;; turtle procedure -- set the appropriate variables to make this turtle sick
to get-bit
  set points points + herbivore-bit-points
end

;;the carnivore's varialbes are changes, after finding an herbivore
to get-to-eat-meat [ eaten ]
  let eat-points (points-of eaten)
  if (eat-points > 0) [
    set points points + carnivore-eat-points
    ask eaten
    [
      set points points + herbivore-bit-points
    ]
 ]
end

to get-to-eat-veggie
  let quantity 0

  set quantity random round (herbivore-eat-points * food  / 10) + 1

  set points points + quantity
  set food food - round (quantity / herbivore-eat-points)
  set pcolor scale-color (green + 2) food 0 100
end


to update-points-monitor
   hubnet-send user-id "Points" (round points)
end


;; when bitten, the herbivore shows a special shape for a very short period of time
to set-starve-shape
  ifelse ( points < start-points )
  [
    if ( shape != ("sick " + base-shape) )
    [ set shape "sick " + base-shape ]
  ]
  [
    ;; we want to check if the turtles shape is already a base-shape
    ;; to prevent flickering in the turles
    if (shape != base-shape)
      [ set shape base-shape ]
  ]
end

to make-students-herbivores
  ask students [
    set carnivore? false
    set label "H"
  ]
end

to make-students-carnivores
  ask students [
    set carnivore? true
    set label "C"
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Quick Start functions ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; instructions to quickly setup the model, and clients to run this activity
to setup-quick-start
  set qs-item 0
  set qs-items
  [
    "Teacher: Follow these directions to run the HubNet activity."
    "Optional: Zoom In (see Tools in the Menu Bar)"
    "Optional: Change any of the settings...."
      "If you do change the settings, press the SETUP button."
    "Press the GO button."
    "Everyone: Open up a HubNet Client on your machine and..."
      "input the IP Address of this computer, type..."
        "your user name in the user name box and press ENTER"
    "Teacher: Have the students move their turtles around to..."
      "acquaint themselves with the interface."
    "Everyone: Watch the plot of the average herbivore and carnivore points."

    "Teacher: To rerun the activity with the same group,..."
      "stop the model by pressing the NetLogo GO button, if it is on."
        "Change any of the settings that you would like."
    "Press the NetLogo RE-RUN button."
    "Teacher: Restart the simulation by pressing the NetLogo GO button again."
      "Infect some turtles and continue."

    "Teacher: To start the simulation over with a new group,..."
      "stop the model by pressing the NetLogo GO button, if it is on..."
        "and follow these instructions again from the beginning."
  ]
  set quick-start (item qs-item qs-items)
end

;; view the next item in the quickstart monitor

to view-next
  set qs-item qs-item + 1
  if qs-item >= length qs-items
  [ set qs-item length qs-items - 1 ]
  set quick-start (item qs-item qs-items)
end

;; view the previous item in the quickstart monitor
to view-prev
  set qs-item qs-item - 1
  if qs-item < 0
  [ set qs-item 0 ]
  set quick-start (item qs-item qs-items)
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Code for interacting with the clients ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; determines which client sent a command, and what the command was
to listen-clients
  while [ hubnet-message-waiting? ]
  [
    hubnet-fetch-message
    ifelse hubnet-enter-message?
    [ create-new-student ]
    [
      ifelse hubnet-exit-message?
      [ remove-student ]
      [ execute-command hubnet-message-tag ]
    ]
  ]
end

;; NetLogo knows what each student turtle is supposed to be
;; doing based on the tag sent by the node:
;; step-size - set the turtle's step-size
;; Up    - make the turtle move up by step-size
;; Down  - make the turtle move down by step-size
;; Right - make the turtle move right by step-size
;; Left  - make the turtle move left by step-size
;; Get a Different Turtle - change the turtle's shape and color
to execute-command [ command ]
  if command = "step-size"
  [
    ask students with [ user-id = hubnet-message-source ]
    [ set step-size hubnet-message ]
  ]
  if command = "Carnivore?" and allow-switching?
  [
    ask students with [ user-id = hubnet-message-source ]
    [
      set carnivore? hubnet-message
      ifelse carnivore?
      [ set label "C" ]
      [ set label "H" ]
    ]
  ]
  if command = "Up"
  [ execute-move 0 ]
  if command = "Down"
  [ execute-move 180 ]
  if command = "Right"
  [ execute-move 90 ]
  if command = "Left"
  [ execute-move 270 ]
  if command = "Change Appearance"
  [ execute-change-turtle ]
end

;; Create a turtle, set its shape, color, and position
;; and tell the node what its turtle looks like and where it is
to create-new-student
  create-custom-students 1
  [
    setup-student-vars
    send-info-to-clients
  ]
end

;; sets the turtle variables to appropriate initial values
to setup-student-vars  ;; turtle procedure
  set user-id hubnet-message-source
  set carnivore? false
  set-unique-shape-and-color
  setxy random screen-size-x random screen-size-y
  set heading random-one-of [0 90 270]
  set label "H"
  set label-color white
  set step-size 1
end

;; pick a base-shape and color for the turtle
to set-unique-shape-and-color
  ifelse carnivore?
  [
    let code random max-possible-carnivore-codes
    while [ member? code used-shape-colors and count students < max-possible-carnivore-codes ]
    [
      set code random max-possible-carnivore-codes
    ]
    set used-shape-colors (lput code used-shape-colors)
    set base-shape item (code mod length carnivore-shape-names) carnivore-shape-names
    set color item (code / length carnivore-shape-names) colors
  ]
  [
    let code random max-possible-herbivore-codes
    while [ member? code used-shape-colors and count students < max-possible-herbivore-codes ]
    [
      set code random max-possible-herbivore-codes
    ]
    set used-shape-colors (lput code used-shape-colors)
    set base-shape item (code mod length herbivore-shape-names) herbivore-shape-names
    set color item (code / length herbivore-shape-names) colors
  ]
  set shape base-shape
  set old-color color
end

;; report the string version of the turtle's color
to-report color-string [ color-value ]
  report item (position color-value colors) color-names
end

;; sends the appropriate monitor information back to the client
to send-info-to-clients
  hubnet-send user-id "You are a:" (color-string color) + " " + base-shape
  hubnet-send user-id "Located at:" ("(" + pxcor + "," + pycor + ")")
end

;; Kill the turtle, set its shape, color, and position
;; and tell the node what its turtle looks like and where it is
to remove-student
  ask students with [ user-id = hubnet-message-source ]
  [
    set used-shape-colors remove my-code used-shape-colors
    die
  ]
end

;; translates a student turtle's shape and color into a code
to-report my-code
  report (position base-shape shape-names) + (length shape-names) * (position color colors)
end

;; Cause the students to move forward step-size in new-heading's heading
to execute-move [ new-heading ]
  ask students with [ user-id = hubnet-message-source ]
  [
    set heading new-heading
    ;; don't allow students to wrap around the screen
    ;; so use move-amount instead of step-size
    jump move-amount
    hubnet-send user-id "Located at:" ("(" + pxcor + "," + pycor + ")")
  ]
end

;; this is called when a hubnet user chooses to change his/her turtle's appearance
to execute-change-turtle
  ask students with [ user-id = hubnet-message-source ]
  [
    set used-shape-colors remove my-code used-shape-colors
    show-turtle
    set-unique-shape-and-color
    hubnet-send user-id "You are a:" (color-string color) + " " + base-shape
  ]
end


; *** NetLogo Model Copyright Notice ***
;
; This activity and associated models and materials was created as part of the projects:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN CLASSROOMS and
; INTEGRATED SIMULATION AND MODELING ENVIRONMENT.
; These projects gratefully acknowledge the support of the
; National Science Foundation (REPP & ROLE programs) -- grant numbers
; REC #9814682 and REC-0126227.
;
; Copyright 2003 by Uri Wilensky. Updated 2004. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from the copyright holders.
; Contact the copyright holders for appropriate licenses for redistribution
; for profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2003).  NetLogo HubNet Herbivore Carnivore model.
; http://ccl.northwestern.edu/netlogo/models/HubNetHerbivoreCarnivore.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2003 Uri Wilensky.  All rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/HubNetHerbivoreCarnivore
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
395
99
825
550
10
10
20.0
1
10
1
1
1
0
0
0
1

CC-WINDOW
5
564
1043
659
Command Center
0

BUTTON
11
10
92
46
Setup
setup-prompt
NIL
1
T
OBSERVER
T
S

BUTTON
199
10
361
56
Go
go
T
1
T
OBSERVER
T
G

BUTTON
9
65
189
98
Create Herbivore Android
create-android-herbivores
NIL
1
T
OBSERVER
T
H

PLOT
14
262
351
459
Total Points
time
points
0.0
25.0
0.0
100.0
true
false
PENS
"meat-avg-points" 1.0 0 -8630108 true
"veggie-avg-points" 1.0 0 -13791810 true

SLIDER
200
105
347
138
android-delay
android-delay
0.0
10.0
0.4
0.1
1
NIL

SWITCH
199
66
348
99
wander?
wander?
0
1
-1000

BUTTON
104
10
189
46
Re-Run
rerun
NIL
1
T
OBSERVER
T
R

BUTTON
14
461
104
494
Clear Plots
setup-plot
NIL
1
T
OBSERVER
T
NIL

BUTTON
397
60
515
93
Reset Instructions
setup-quick-start
NIL
1
T
OBSERVER
T
NIL

BUTTON
732
60
816
93
NEXT >>>
view-next
NIL
1
T
OBSERVER
T
NIL

BUTTON
655
60
733
93
<<< PREV
view-prev
NIL
1
T
OBSERVER
T
NIL

MONITOR
397
10
817
59
Quick Start Instructions- More in Info Window
quick-start
0
1

BUTTON
9
107
189
140
Create Carnivore Android
create-android-carnivores
NIL
1
T
OBSERVER
T
C

SLIDER
834
101
1025
134
carnivore-eat-points
carnivore-eat-points
0
200
50
1
1
NIL

SLIDER
835
136
1026
169
herbivore-eat-points
herbivore-eat-points
1
100
10
1
1
NIL

SLIDER
834
171
1026
204
herbivore-bit-points
herbivore-bit-points
-500
0
-50
1
1
NIL

SLIDER
19
190
191
223
plant-regrowth-rate
plant-regrowth-rate
0
10
0.9
0.1
1
NIL

SLIDER
834
203
1034
236
herbivore-starve-penalty
herbivore-starve-penalty
0
10
5.0
0.1
1
%

SLIDER
834
236
1034
269
carnivore-starve-penalty
carnivore-starve-penalty
0
10
1.0
0.1
1
%

MONITOR
835
287
908
336
carnivores
count turtles with [ carnivore? ]
0
1

MONITOR
835
343
908
392
herbivores
count turtles with [ not carnivore? ]
3
1

SWITCH
21
153
190
186
allow-switching?
allow-switching?
0
1
-1000

BUTTON
199
153
375
186
NIL
make-students-herbivores
NIL
1
T
OBSERVER
T
NIL

BUTTON
199
191
375
224
NIL
make-students-carnivores
NIL
1
T
OBSERVER
T
NIL

@#$#@#$#@
WHAT IS IT?
-----------
This model simulates the relationship between herbivores and carnivores.  This population can consist of either students, which are turtles controlled by individual students via the HubNet Client, or turtles that are generated and controlled by NetLogo, called androids, or both androids and students.

Turtles move around, searching for food.  Herbivores search for plants (green patches), while carnivores eat by finding other herbivores. Each animal loses points each turn that must be replenished by eating.  Herbivores may lose points whenever getting "bitten", that is, when coming into contact with a carnivore, which also results in increased points for the carnivore. When an animal is starving, it shows a red dot.

For further documentation, see the Participatory Simulations Guide found at http://ccl.northwestern.edu/ps/


HOW TO USE IT
-------------
Quickstart Instructions:
------------------------
Teacher: Follow these directions to run the HubNet activity.
Optional: Zoom In (see Tools in the Menu Bar)
Optional: Change any of the settings.  If you do change the settings, press the SETUP button.  Press the GO button to start the model.
Everyone: Open up a HubNet Client on your machine and input the IP Address of this computer, type your user name in the user name box and press ENTER.

Teacher: Have the students move their turtles around to acquaint themselves with the interface. Have the students all start as herbivores by first setting "allow-switching?" to OFF and hitting the "make-students-herbivores" button.

Teacher: To start the simulation over with a new group, stop the model by pressing the NetLogo GO button, if it is on and follow these instructions again from the beginning.

Buttons:
--------
SETUP - clears all turtles and patches and the plot.  This should only be pressed when starting out with a new group of users since all data is lost.
GO - runs the simulation
RE-RUN - resets all turtles.  This should be used to setup the model again for collecting more data or running the model again with the same users connected.
CREATE ANDROID HERBIVORE - adds randomly moving herbivore turtle to the simulation.
CREATE ANDROID CARNIVORE - adds randomly moving carnivore turtle to the simulation.

NEXT >>> - shows the next quick start instruction
<<< PREVIOUS - shows the previous quick start instruction
RESET INSTRUCTIONS - shows the first quick start instruction

Sliders:
--------
PLANT-REGROWTH-RATE - the delay time, in seconds, for plants to regrow, providing more food for herbivores
ANDROID-DELAY - the delay time, in seconds, for android movement - the higher the number, the slower the androids move
CARNIVORE-EAT-POINTS - the number of points a carnivore will get every time it can eat
HERBIVORE-EAT-POINTS - the number of points an herbivore will get every time it can eat
HERBIVORE-STARVE-PENALTY - the percentage of points an herbivore will lose every turn.
CARNIVORE-STARVE-PENALTY - the percentage of points a carnivore will lose every turn.
HERBIVORE-BIT-POINTS - the number of points an herbivore will lose if a carnivore gets some food out of it!

Switches:
---------
PLANTS-REGROW? - when on, the patches will regrow plants, that is, will gain more food points as time goes on
ALLOW-SWITCHING? - when off, students remain as they are, either herbivores or carnivores. when this switch is on, the students can switch between acting as a herbivores or carnivores.

Monitors:
---------
MEAN CARNIVORE POINTS - the number of points accumulated by all carnivores
MEAN HERBIVORE POINTS - the number of points accumulated by all herbivores
CARNIVORES - the number of current carnivores
HERBIVORES - the number of current herbivores

Plots:
------
Total Points- shows the number of collected points over time, comparing herbivores and carnivores

Client Information
------------------
After logging in, the client interface will appear for the students, and if GO is pressed in NetLogo they will be assigned a turtle which will be described in the YOU ARE A: monitor.  And their current location will be shown in the LOCATED AT: monitor.  If the student doesn't like their assigned shape and/or color they can hit the CHANGE APPEARANCE button at any time to change to another random appearance.

The student controls the movement of their turtle with the UP, DOWN, LEFT, and RIGHT buttons and the STEP-SIZE slider.  Clicking any of the directional buttons will cause their turtle to move in the respective direction a distance of STEP-SIZE.

The students can watch the progress of the disease in the NUMBER SICK plot which is identical to the plot of the same name in NetLogo.


THINGS TO NOTICE
----------------
Android sheep tend to go toward areas of greater food.  In this model, carnivores know to go to those locations as well.

Students, when given the opportunity to change from herbivore to carnivore, may find it quite beneficial to make a change to carnivore.  However, too many students may realize this, make the change, and ultimately starve.  Students will have to figure out that what may seem beneficial to them will ultimately hurt themselves, as well as everyone else in the environment.

Carnivores may find it better to form packs.  Herbivores, however, may learn to stay more scattered, so as to not become easy prey to a large group of carnivores.


THINGS TO TRY
-------------
Use the model with the entire class to serve as an introduction to the topic.  Then have students use the NetLogo model individually, in a computer lab, to explore the effects of the various parameters.  Discuss what they find, observe, and can conclude from this model.

Try changing the amount of points that a carnivore or herbivore gets for eating or being eaten. You will find that it change change your results dramatically. Also, try changing the percentage of energy that animals expend each turn.


EXTENDING THE MODEL
-------------------
Try making the androids smarter.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2004).  HubNet Herbivore Carnivore model. http://ccl.northwestern.edu/netlogo/models/HubNetHerbivoreCarnivore. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
false
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

android
false
0
Rectangle -7500403 true true 105 74 210 239
Polygon -7500403 true true 104 78 34 129 47 148 114 89
Polygon -7500403 true true 198 81 274 108 258 142 192 104
Polygon -7500403 true true 115 239 115 289 133 289 133 237
Polygon -7500403 true true 176 235 176 287 192 287 192 234
Rectangle -7500403 true true 119 12 194 73
Rectangle -16777216 true false 129 22 147 36
Rectangle -16777216 true false 164 23 184 37
Rectangle -16777216 true false 151 113 163 125
Rectangle -16777216 true false 153 142 164 154
Rectangle -16777216 true false 154 171 166 184

bear
false
0
Rectangle -7500403 true true 106 35 181 92
Circle -7500403 true true 110 17 18
Circle -7500403 true true 128 17 19
Rectangle -7500403 true true 80 58 111 89
Rectangle -7500403 true true 98 95 101 95
Rectangle -7500403 true true 92 93 192 104
Rectangle -7500403 true true 83 101 199 114
Rectangle -7500403 true true 78 107 205 130
Rectangle -7500403 true true 74 122 209 240
Rectangle -7500403 true true 77 236 204 249
Rectangle -7500403 true true 106 89 180 104
Rectangle -7500403 true true 80 247 198 258
Rectangle -7500403 true true 85 255 108 281
Rectangle -7500403 true true 169 252 195 283
Rectangle -7500403 true true 158 276 171 283
Rectangle -7500403 true true 74 276 90 281
Circle -7500403 true true 188 201 35
Circle -16777216 true false 111 40 13
Rectangle -7500403 true true 51 163 87 184
Rectangle -7500403 true true 45 166 53 169
Rectangle -7500403 true true 42 172 52 173
Rectangle -7500403 true true 38 178 53 183
Circle -7500403 true true 72 62 20

butterfly
false
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7500403 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7500403 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7500403 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7500403 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7500403 true 167 47 159 82
Line -7500403 true 136 47 145 81
Circle -7500403 true true 165 45 8
Circle -7500403 true true 134 45 6
Circle -7500403 true true 133 44 7
Circle -7500403 true true 133 43 8

cat
false
0
Line -7500403 true 285 240 210 240
Line -7500403 true 195 300 165 255
Line -7500403 true 15 240 90 240
Line -7500403 true 285 285 195 240
Line -7500403 true 105 300 135 255
Line -16777216 false 150 270 150 285
Line -16777216 false 15 75 15 120
Polygon -7500403 true true 300 15 285 30 255 30 225 75 195 60 255 15
Polygon -7500403 true true 285 135 210 135 180 150 180 45 285 90
Polygon -7500403 true true 120 45 120 210 180 210 180 45
Polygon -7500403 true true 180 195 165 300 240 285 255 225 285 195
Polygon -7500403 true true 180 225 195 285 165 300 150 300 150 255 165 225
Polygon -7500403 true true 195 195 195 165 225 150 255 135 285 135 285 195
Polygon -7500403 true true 15 135 90 135 120 150 120 45 15 90
Polygon -7500403 true true 120 195 135 300 60 285 45 225 15 195
Polygon -7500403 true true 120 225 105 285 135 300 150 300 150 255 135 225
Polygon -7500403 true true 105 195 105 165 75 150 45 135 15 135 15 195
Polygon -7500403 true true 285 120 270 90 285 15 300 15
Line -7500403 true 15 285 105 240
Polygon -7500403 true true 15 120 30 90 15 15 0 15
Polygon -7500403 true true 0 15 15 30 45 30 75 75 105 60 45 15
Line -16777216 false 164 262 209 262
Line -16777216 false 223 231 208 261
Line -16777216 false 136 262 91 262
Line -16777216 false 77 231 92 261

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

dog
false
0
Polygon -7500403 true true 300 165 300 195 270 210 183 204 180 240 165 270 165 300 120 300 0 240 45 165 75 90 75 45 105 15 135 45 165 45 180 15 225 15 255 30 225 30 210 60 225 90 225 105
Polygon -16777216 true false 0 240 120 300 165 300 165 285 120 285 10 221
Line -16777216 false 210 60 180 45
Line -16777216 false 90 45 90 90
Line -16777216 false 90 90 105 105
Line -16777216 false 105 105 135 60
Line -16777216 false 90 45 135 60
Line -16777216 false 135 60 135 45
Line -16777216 false 181 203 151 203
Line -16777216 false 150 201 105 171
Circle -16777216 true false 171 88 34
Circle -16777216 false false 261 162 30

hawk
true
0
Polygon -7500403 true true 151 170 136 170 123 229 143 244 156 244 179 229 166 170
Polygon -16777216 true false 152 154 137 154 125 213 140 229 159 229 179 214 167 154
Polygon -7500403 true true 151 140 136 140 126 202 139 214 159 214 176 200 166 140
Polygon -16777216 true false 151 125 134 124 128 188 140 198 161 197 174 188 166 125
Polygon -7500403 true true 152 86 227 72 286 97 272 101 294 117 276 118 287 131 270 131 278 141 264 138 267 145 228 150 153 147
Polygon -7500403 true true 160 74 159 61 149 54 130 53 139 62 133 81 127 113 129 149 134 177 150 206 168 179 172 147 169 111
Circle -16777216 true false 144 55 7
Polygon -16777216 true false 129 53 135 58 139 54
Polygon -7500403 true true 148 86 73 72 14 97 28 101 6 117 24 118 13 131 30 131 22 141 36 138 33 145 72 150 147 147

horse
false
0
Polygon -7500403 true true 226 121 90 121 60 136 61 181 106 195 210 195 255 181 255 135
Rectangle -7500403 true true 61 180 75 268
Rectangle -7500403 true true 89 187 104 275
Rectangle -7500403 true true 210 186 225 269
Rectangle -7500403 true true 241 178 254 275
Polygon -7500403 true true 62 138 60 107 23 107 17 102 18 85 24 82 54 79 66 63 87 64 94 128
Circle -16777216 true false 63 72 12
Polygon -7500403 true true 226 120 278 138 284 185 265 188 260 142
Polygon -7500403 true true 65 65 70 50
Polygon -7500403 true true 65 67 74 46 79 65 93 49 88 73 65 67

lobster
false
0
Polygon -7500403 true true 105 106 122 238 100 259 100 276 118 275 150 240 180 275 197 275 197 258 174 238 196 105 178 77 123 77 105 106
Polygon -7500403 true true 112 103 84 78 105 60 94 14 85 57 78 19 67 59 76 83 110 118
Polygon -7500403 true true 192 120 228 85 241 57 231 22 223 59 217 17 205 61 216 79 192 108
Rectangle -7500403 true true 71 125 117 133
Rectangle -7500403 true true 73 149 119 157
Rectangle -7500403 true true 78 178 123 187
Rectangle -7500403 true true 183 131 219 138
Rectangle -7500403 true true 180 152 216 160
Rectangle -7500403 true true 173 176 217 184
Rectangle -7500403 true true 127 56 136 82
Rectangle -7500403 true true 160 55 169 81

monster
false
0
Polygon -7500403 true true 75 150 90 195 210 195 225 150 255 120 255 45 180 0 120 0 45 45 45 120
Circle -16777216 true false 165 60 60
Circle -16777216 true false 75 60 60
Polygon -7500403 true true 225 150 285 195 285 285 255 300 255 210 180 165
Polygon -7500403 true true 75 150 15 195 15 285 45 300 45 210 120 165
Polygon -7500403 true true 210 210 225 285 195 285 165 165
Polygon -7500403 true true 90 210 75 285 105 285 135 165
Rectangle -7500403 true true 135 165 165 270

moose
false
0
Polygon -7500403 true true 226 121 90 121 60 136 61 181 106 195 210 195 255 181 255 135
Rectangle -7500403 true true 61 180 75 268
Rectangle -7500403 true true 89 187 104 275
Rectangle -7500403 true true 210 186 225 269
Rectangle -7500403 true true 241 178 254 275
Polygon -7500403 true true 62 138 60 107 23 107 17 102 18 85 24 82 54 79 66 63 87 64 94 128
Circle -16777216 true false 63 72 12
Polygon -7500403 true true 226 120 278 138 284 185 265 188 260 142
Polygon -7500403 true true 65 65 70 50
Polygon -7500403 true true 68 64 60 49 17 49 7 41 6 12 22 40 38 40 35 14 49 39 66 39 64 16 78 66 68 64
Polygon -7500403 true true 76 64 97 17 95 38 111 38 122 16 121 38 136 38 152 11 151 38 144 47 97 47 86 66

rabbit
false
0
Polygon -7500403 true true 61 150 76 180 91 195 103 214 91 240 76 255 61 270 76 270 106 255 132 209 151 210 181 210 211 240 196 255 181 255 166 247 151 255 166 270 211 270 241 255 240 210 270 225 285 165 256 135 226 105 166 90 91 105
Polygon -7500403 true true 75 164 94 104 70 82 45 89 19 104 4 149 19 164 37 162 59 153
Polygon -7500403 true true 64 98 96 87 138 26 130 15 97 36 54 86
Polygon -7500403 true true 49 89 57 47 78 4 89 20 70 88
Circle -16777216 true false 37 103 16
Line -16777216 false 44 150 104 150
Line -16777216 false 39 158 84 175
Line -16777216 false 29 159 57 195
Polygon -5825686 true false 0 150 15 165 15 150
Polygon -5825686 true false 76 90 97 47 130 32
Line -16777216 false 180 210 165 180
Line -16777216 false 165 180 180 165
Line -16777216 false 180 165 225 165
Line -16777216 false 180 210 210 240

shark
false
0
Polygon -7500403 true true 283 153 288 149 271 146 301 145 300 138 247 119 190 107 104 117 54 133 39 134 10 99 9 112 19 142 9 175 10 185 40 158 69 154 64 164 80 161 86 156 132 160 209 164
Polygon -7500403 true true 199 161 152 166 137 164 169 154
Polygon -7500403 true true 188 108 172 83 160 74 156 76 159 97 153 112
Circle -16777216 true false 256 129 12
Line -16777216 false 222 134 222 150
Line -16777216 false 217 134 217 150
Line -16777216 false 212 134 212 150
Polygon -7500403 true true 78 125 62 118 63 130
Polygon -7500403 true true 121 157 105 161 101 156 106 152

sheep
false
0
Rectangle -7500403 true true 30 75 210 225
Circle -7500403 true true 135 75 150
Rectangle -16777216 true false 166 225 219 286
Rectangle -16777216 true false 62 225 120 285
Circle -16777216 true false 207 88 92

sick android
false
0
Rectangle -7500403 true true 105 74 210 239
Polygon -7500403 true true 104 78 34 129 47 148 114 89
Polygon -7500403 true true 198 81 274 108 258 142 192 104
Polygon -7500403 true true 115 239 115 289 133 289 133 237
Polygon -7500403 true true 176 235 176 287 192 287 192 234
Rectangle -7500403 true true 119 12 194 73
Rectangle -16777216 true false 129 22 147 36
Rectangle -16777216 true false 164 23 184 37
Rectangle -16777216 true false 151 113 163 125
Rectangle -16777216 true false 153 142 164 154
Rectangle -16777216 true false 154 171 166 184
Circle -2674135 true false 150 150 90

sick bear
false
0
Rectangle -7500403 true true 106 35 181 92
Circle -7500403 true true 110 17 18
Circle -7500403 true true 128 17 19
Rectangle -7500403 true true 80 58 111 89
Rectangle -7500403 true true 98 95 101 95
Rectangle -7500403 true true 92 93 192 104
Rectangle -7500403 true true 83 101 199 114
Rectangle -7500403 true true 78 107 205 130
Rectangle -7500403 true true 74 122 209 240
Rectangle -7500403 true true 77 236 204 249
Rectangle -7500403 true true 106 89 180 104
Rectangle -7500403 true true 80 247 198 258
Rectangle -7500403 true true 85 255 108 281
Rectangle -7500403 true true 169 252 195 283
Rectangle -7500403 true true 158 276 171 283
Rectangle -7500403 true true 74 276 90 281
Circle -7500403 true true 188 201 35
Circle -16777216 true false 111 40 13
Rectangle -7500403 true true 51 163 87 184
Rectangle -7500403 true true 45 166 53 169
Rectangle -7500403 true true 42 172 52 173
Rectangle -7500403 true true 38 178 53 183
Circle -7500403 true true 72 62 20
Circle -2674135 true false 118 163 95

sick butterfly
false
0
Rectangle -7500403 true true 92 135 207 224
Circle -7500403 true true 158 53 134
Circle -7500403 true true 165 180 90
Circle -7500403 true true 45 180 90
Circle -7500403 true true 8 53 134
Line -16777216 false 43 189 253 189
Rectangle -7500403 true true 135 60 165 285
Circle -7500403 true true 165 15 30
Circle -7500403 true true 105 15 30
Line -7500403 true 120 30 135 60
Line -7500403 true 165 60 180 30
Line -16777216 false 135 60 135 285
Line -16777216 false 165 285 165 60
Circle -2674135 true false 156 171 108

sick cat
false
0
Line -7500403 true 285 240 210 240
Line -7500403 true 195 300 165 255
Line -7500403 true 15 240 90 240
Line -7500403 true 285 285 195 240
Line -7500403 true 105 300 135 255
Line -16777216 false 150 270 150 285
Line -16777216 false 15 75 15 120
Polygon -7500403 true true 300 15 285 30 255 30 225 75 195 60 255 15
Polygon -7500403 true true 285 135 210 135 180 150 180 45 285 90
Polygon -7500403 true true 120 45 120 210 180 210 180 45
Polygon -7500403 true true 180 195 165 300 240 285 255 225 285 195
Polygon -7500403 true true 180 225 195 285 165 300 150 300 150 255 165 225
Polygon -7500403 true true 195 195 195 165 225 150 255 135 285 135 285 195
Polygon -7500403 true true 15 135 90 135 120 150 120 45 15 90
Polygon -7500403 true true 120 195 135 300 60 285 45 225 15 195
Polygon -7500403 true true 120 225 105 285 135 300 150 300 150 255 135 225
Polygon -7500403 true true 105 195 105 165 75 150 45 135 15 135 15 195
Polygon -7500403 true true 285 120 270 90 285 15 300 15
Line -7500403 true 15 285 105 240
Polygon -7500403 true true 15 120 30 90 15 15 0 15
Polygon -7500403 true true 0 15 15 30 45 30 75 75 105 60 45 15
Line -16777216 false 164 262 209 262
Line -16777216 false 223 231 208 261
Line -16777216 false 136 262 91 262
Line -16777216 false 77 231 92 261
Circle -2674135 true false 0 195 90

sick cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123
Circle -2674135 true false 148 148 95

sick dog
false
0
Polygon -7500403 true true 300 165 300 195 270 210 183 204 180 240 165 270 165 300 120 300 0 240 45 165 75 90 75 45 105 15 135 45 165 45 180 15 225 15 255 30 225 30 210 60 225 90 225 105
Polygon -16777216 true false 0 240 120 300 165 300 165 285 120 285 10 221
Line -16777216 false 210 60 180 45
Line -16777216 false 90 45 90 90
Line -16777216 false 90 90 105 105
Line -16777216 false 105 105 135 60
Line -16777216 false 90 45 135 60
Line -16777216 false 135 60 135 45
Line -16777216 false 181 203 151 203
Line -16777216 false 150 201 105 171
Circle -16777216 true false 171 88 34
Circle -16777216 false false 261 162 30
Circle -2674135 true false -2 178 95

sick hawk
true
0
Polygon -7500403 true true 151 170 136 170 123 229 143 244 156 244 179 229 166 170
Polygon -16777216 true false 152 154 137 154 125 213 140 229 159 229 179 214 167 154
Polygon -7500403 true true 151 140 136 140 126 202 139 214 159 214 176 200 166 140
Polygon -16777216 true false 151 125 134 124 128 188 140 198 161 197 174 188 166 125
Polygon -7500403 true true 152 86 227 72 286 97 272 101 294 117 276 118 287 131 270 131 278 141 264 138 267 145 228 150 153 147
Polygon -7500403 true true 160 74 159 61 149 54 130 53 139 62 133 81 127 113 129 149 134 177 150 206 168 179 172 147 169 111
Circle -16777216 true false 144 55 7
Polygon -16777216 true false 129 53 135 58 139 54
Polygon -7500403 true true 148 86 73 72 14 97 28 101 6 117 24 118 13 131 30 131 22 141 36 138 33 145 72 150 147 147
Circle -2674135 true false 90 105 60

sick horse
false
0
Polygon -7500403 true true 226 121 90 121 60 136 61 181 106 195 210 195 255 181 255 135
Rectangle -7500403 true true 61 180 75 268
Rectangle -7500403 true true 89 187 104 275
Rectangle -7500403 true true 210 186 225 269
Rectangle -7500403 true true 241 178 254 275
Polygon -7500403 true true 62 138 60 107 23 107 17 102 18 85 24 82 54 79 66 63 87 64 94 128
Circle -16777216 true false 63 72 12
Polygon -7500403 true true 226 120 278 138 284 185 265 188 260 142
Polygon -7500403 true true 65 65 70 50
Polygon -7500403 true true 65 67 74 46 79 65 93 49 88 73 65 67
Circle -2674135 true false 75 105 90

sick lobster
false
0
Polygon -7500403 true true 105 106 122 238 100 259 100 276 118 275 150 240 180 275 197 275 197 258 174 238 196 105 178 77 123 77 105 106
Polygon -7500403 true true 112 103 84 78 105 60 94 14 85 57 78 19 67 59 76 83 110 118
Polygon -7500403 true true 192 120 228 85 241 57 231 22 223 59 217 17 205 61 216 79 192 108
Rectangle -7500403 true true 71 125 117 133
Rectangle -7500403 true true 73 149 119 157
Rectangle -7500403 true true 78 178 123 187
Rectangle -7500403 true true 183 131 219 138
Rectangle -7500403 true true 180 152 216 160
Rectangle -7500403 true true 173 176 217 184
Rectangle -7500403 true true 127 56 136 82
Rectangle -7500403 true true 160 55 169 81
Circle -2674135 true false 131 176 67

sick monster
false
0
Polygon -7500403 true true 75 150 90 195 210 195 225 150 255 120 255 45 180 0 120 0 45 45 45 120
Circle -16777216 true false 165 60 60
Circle -16777216 true false 75 60 60
Polygon -7500403 true true 225 150 285 195 285 285 255 300 255 210 180 165
Polygon -7500403 true true 75 150 15 195 15 285 45 300 45 210 120 165
Polygon -7500403 true true 210 210 225 285 195 285 165 165
Polygon -7500403 true true 90 210 75 285 105 285 135 165
Rectangle -7500403 true true 135 165 165 270
Circle -2674135 true false -2 163 95

sick moose
false
0
Polygon -7500403 true true 226 121 90 121 60 136 61 181 106 195 210 195 255 181 255 135
Rectangle -7500403 true true 61 180 75 268
Rectangle -7500403 true true 89 187 104 275
Rectangle -7500403 true true 210 186 225 269
Rectangle -7500403 true true 241 178 254 275
Polygon -7500403 true true 62 138 60 107 23 107 17 102 18 85 24 82 54 79 66 63 87 64 94 128
Circle -16777216 true false 63 72 12
Polygon -7500403 true true 226 120 278 138 284 185 265 188 260 142
Polygon -7500403 true true 65 65 70 50
Polygon -7500403 true true 68 64 60 49 17 49 7 41 6 12 22 40 38 40 35 14 49 39 66 39 64 16 78 66 68 64
Polygon -7500403 true true 76 64 97 17 95 38 111 38 122 16 121 38 136 38 152 11 151 38 144 47 97 47 86 66
Circle -2674135 true false 79 87 115

sick rabbit
false
0
Polygon -7500403 true true 61 150 76 180 91 195 103 214 91 240 76 255 61 270 76 270 106 255 132 209 151 210 181 210 211 240 196 255 181 255 166 247 151 255 166 270 211 270 241 255 240 210 270 225 285 165 256 135 226 105 166 90 91 105
Polygon -7500403 true true 75 164 94 104 70 82 45 89 19 104 4 149 19 164 37 162 59 153
Polygon -7500403 true true 64 98 96 87 138 26 130 15 97 36 54 86
Polygon -7500403 true true 49 89 57 47 78 4 89 20 70 88
Circle -16777216 true false 37 103 16
Line -16777216 false 44 150 104 150
Line -16777216 false 39 158 84 175
Line -16777216 false 29 159 57 195
Polygon -5825686 true false 0 150 15 165 15 150
Polygon -5825686 true false 76 90 97 47 130 32
Line -16777216 false 180 210 165 180
Line -16777216 false 165 180 180 165
Line -16777216 false 180 165 225 165
Line -16777216 false 180 210 210 240
Circle -2674135 true false 193 193 95

sick shark
false
0
Polygon -7500403 true true 283 153 288 149 271 146 301 145 300 138 247 119 190 107 104 117 54 133 39 134 10 99 9 112 19 142 9 175 10 185 40 158 69 154 64 164 80 161 86 156 132 160 209 164
Polygon -7500403 true true 199 161 152 166 137 164 169 154
Polygon -7500403 true true 188 108 172 83 160 74 156 76 159 97 153 112
Circle -16777216 true false 256 129 12
Line -16777216 false 222 134 222 150
Line -16777216 false 217 134 217 150
Line -16777216 false 212 134 212 150
Polygon -7500403 true true 78 125 62 118 63 130
Polygon -7500403 true true 121 157 105 161 101 156 106 152
Circle -2674135 true false 120 120 60

sick sheep
false
0
Rectangle -7500403 true true 30 75 210 225
Circle -7500403 true true 135 75 150
Rectangle -16777216 true false 166 225 219 286
Rectangle -16777216 true false 62 225 120 285
Circle -16777216 true false 207 88 92
Circle -2674135 true false 178 193 95

sick squirrel
false
0
Polygon -7500403 true true 87 267 106 290 145 292 157 288 175 292 209 292 207 281 190 276 174 277 156 271 154 261 157 245 151 230 156 221 171 209 214 165 231 171 239 171 263 154 281 137 294 136 297 126 295 119 279 117 241 145 242 128 262 132 282 124 288 108 269 88 247 73 226 72 213 76 208 88 190 112 151 107 119 117 84 139 61 175 57 210 65 231 79 253 65 243 46 187 49 157 82 109 115 93 146 83 202 49 231 13 181 12 142 6 95 30 50 39 12 96 0 162 23 250 68 275
Polygon -16777216 true false 237 85 249 84 255 92 246 95
Line -16777216 false 221 82 213 93
Line -16777216 false 253 119 266 124
Line -16777216 false 278 110 278 116
Line -16777216 false 149 229 135 211
Line -16777216 false 134 211 115 207
Line -16777216 false 117 207 106 211
Line -16777216 false 91 268 131 290
Line -16777216 false 220 82 213 79
Line -16777216 false 286 126 294 128
Line -16777216 false 193 284 206 285
Circle -2674135 true false 103 178 95

sick turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99
Circle -2674135 true false 165 150 90

sick wolf
false
0
Rectangle -7500403 true true 15 105 105 165
Rectangle -7500403 true true 45 90 105 105
Polygon -7500403 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7500403 true true 105 120 263 195
Rectangle -7500403 true true 108 195 259 201
Rectangle -7500403 true true 114 201 252 210
Rectangle -7500403 true true 120 210 243 214
Rectangle -7500403 true true 115 114 255 120
Rectangle -7500403 true true 128 108 248 114
Rectangle -7500403 true true 150 105 225 108
Rectangle -7500403 true true 132 214 155 270
Rectangle -7500403 true true 110 260 132 270
Rectangle -7500403 true true 210 214 232 270
Rectangle -7500403 true true 189 260 210 270
Line -7500403 true 263 127 281 155
Line -7500403 true 281 155 281 192
Circle -2674135 true false 75 180 90

squirrel
false
0
Polygon -7500403 true true 87 267 106 290 145 292 157 288 175 292 209 292 207 281 190 276 174 277 156 271 154 261 157 245 151 230 156 221 171 209 214 165 231 171 239 171 263 154 281 137 294 136 297 126 295 119 279 117 241 145 242 128 262 132 282 124 288 108 269 88 247 73 226 72 213 76 208 88 190 112 151 107 119 117 84 139 61 175 57 210 65 231 79 253 65 243 46 187 49 157 82 109 115 93 146 83 202 49 231 13 181 12 142 6 95 30 50 39 12 96 0 162 23 250 68 275
Polygon -16777216 true false 237 85 249 84 255 92 246 95
Line -16777216 false 221 82 213 93
Line -16777216 false 253 119 266 124
Line -16777216 false 278 110 278 116
Line -16777216 false 149 229 135 211
Line -16777216 false 134 211 115 207
Line -16777216 false 117 207 106 211
Line -16777216 false 91 268 131 290
Line -16777216 false 220 82 213 79
Line -16777216 false 286 126 294 128
Line -16777216 false 193 284 206 285

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wolf
false
0
Rectangle -7500403 true true 15 105 105 165
Rectangle -7500403 true true 45 90 105 105
Polygon -7500403 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7500403 true true 105 120 263 195
Rectangle -7500403 true true 108 195 259 201
Rectangle -7500403 true true 114 201 252 210
Rectangle -7500403 true true 120 210 243 214
Rectangle -7500403 true true 115 114 255 120
Rectangle -7500403 true true 128 108 248 114
Rectangle -7500403 true true 150 105 225 108
Rectangle -7500403 true true 132 214 155 270
Rectangle -7500403 true true 110 260 132 270
Rectangle -7500403 true true 210 214 232 270
Rectangle -7500403 true true 189 260 210 270
Line -7500403 true 263 127 281 155
Line -7500403 true 281 155 281 192

@#$#@#$#@
NetLogo 3.0beta6
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
