__extensions [ "sound.jar" ]

globals [
  old-selected
  selected
  dragged
  ticks
  K
  caption-patch
]

breeds [ levers ]

levers-own [
  cx cy
  x1 x2 y1 y2
  hook1
  hook2
  xv yv av
  fixed? -f
  spin?  -r
  armlen
  from-h1 to-h2
  h1x2 h1y2
  h2x1 h2y1
  h1cx h1cy
  h2cx h2cy
  ideal-dist
  ideal-angl
  ratio1
  ratio2
  inst
]

;;;
;;; SETUP PROCEDURES
;;;

to setup
  clear-all
  set-default-shape levers "lever"
  set k 1
  set ticks 0
  set caption-patch patch screen-edge-x screen-edge-y
  create-custom-levers num-levers [
    initialize-lever
  ]
end
  
to initialize-lever  ;; lever procedure
  reset-lever
  set heading 180
  setxy 0 (((who / num-levers) * screen-size-y - screen-edge-y ) * -0.8)
  set cx xcor
  set cy ycor
  ifelse who = 0
  [ set hook1 self
    set fixed? true
  ]
  [ set hook1 turtle (who - 1)
  ]
  ifelse who = num-levers - 1
  [ set hook2 self
    set label "2"
    set fixed? false
  ]
  [ set hook2 turtle ( who + 1 )
  ]
  set x1 xcor - dx * scale * armlen
  set y1 ycor - dy * scale * armlen
  set x2 xcor + dx * scale * armlen
  set y2 ycor + dy * scale * armlen
  set inst item 0 instruments
end

to setup-dangle
  setup
  ask turtle (num-levers - 4) [
    set armlen armlen * 2
    set-spin==>on-cw
  ]
  setup-finish
end

to setup-finish
  select-last-lever
  ask levers [ lever-display ] 
end

to setup-chaos-tentacle
  setup
  ask levers
    [ set armlen (num-levers - who ) * .1
      set-fixed==>off
      ifelse who mod 2 = 0
      [ set-spin==>on-cw ]
      [ set-spin==>on-ws ]
    ]
  ask min-one-of levers [who]
    [ set cx 0
      set cy 0
      setxy cx cy
      set-fixed==>on
    ]
  setup-finish
end

to setup-crazy-machine
  setup
  ask levers
    [ ; choose random location, in the inner area of the screen
      set cx (random-float 1.6 - .8 ) * screen-edge-x
      set cy (random-float 1.6 - .8 ) * screen-edge-y
      set xv 0
      set yv 0
      setxy cx cy
      ifelse ( random 5 = 0 ) [ set-fixed==>on ] [ set-fixed==>off ]
      set spin? random 3 - 1
      ifelse spin? = 0    [ set-spin==>off ]
      [ ifelse spin? = 1  [ set-spin==>on-cw  ]
                          [ set-spin==>on-ws  ]
      ]
      set armlen precision ( .5 + .5 * random 4 ) 1
      lever-display
    ]
  setup-finish
end

;;;

to-report mean-heading [ a b ]
  report atan (( sin a + sin b) * .5)
	      (( cos a + cos b) * .5)
end

;; reports a weighted mean of the given headings
to-report mean-heading-w [ a b c w]
  let w/2 (1 - w) * .5
  let xx (( w/2 * sin a + w/2 * sin b + w * sin c ) )
  let yy (( w/2 * cos a + w/2 * cos b + w * cos c ) )
  ifelse xx = 0 and yy = 0
  [ report 0 ]
  [ report atan (( w/2 * sin a + w/2 * sin b + w * sin c ) )
		(( w/2 * cos a + w/2 * cos b + w * cos c ) ) ]
end

to reset-lever
  set-fixed==>off
  set-spin==>off
  set armlen .5
  set size scale
end

;;;
;;; RUNTIME PROCEDURES
;;;

to go
  set ticks ticks + 1
  ask levers
  [ monitor-mouse
    lever-display
  ]
  ask levers
  [ calc-arm-coords
    calc-hook-coords
  ]
  ask levers
  [ lever! ]
end

to-report sign [ a ]
  ifelse a = 0
  [ report 0 ]
  [  report a / abs a ]
end

to monitor-mouse
  let mouse-here? abs (mouse-xcor - xcor)  < scale / 2 and abs ( mouse-ycor - ycor) < scale / 2
  ifelse mouse-here?
  [ if color != white [ set color white ] ]
  [ if color = white [ set color (5 + (10 * (who mod 14 ))) ] ]
  ifelse mouse-down?
  [ if dragged = nobody and mouse-here?
    [ set dragged self
      set selected self
    ]
  ]
  [ set dragged nobody ]
end

to lever-display
  let -s ""
  if size != armlen * 2 * scale
  [ set size scale * 2 * armlen ]
  ifelse selected = self
  [ set -s "-s"
    if selected != old-selected
    [ set plabel-of caption-patch (word "selected: " who " - " armlen " ")
      set label (word who " - " armlen " "  " ---->     ")
    ]
  ]
  [ if label != no-label
    [ set label no-label ]
  ]
  set shape (word "lever" -f -r -s)
end

to calc-arm-coords
  set x1 cx - dx * scale * armlen
  set y1 cy - dy * scale * armlen
  set x2 cx + dx * scale * armlen
  set y2 cy + dy * scale * armlen
end

to calc-hook-coords
  ifelse hook1 = self
  [ set h1x2 x1
    set h1y2 y1
    set h1cx cx - dx * scale * armlen
    set h1cy cy - dy * scale * armlen
  ]
  [ set h1x2 x2-of hook1
    set h1y2 y2-of hook1
    set h1cx cx-of hook1
    set h1cy cy-of hook1
  ]
  ifelse hook2 = self
  [ set h2x1 x2
    set h2y1 y2
    set h2cx cx + scale * dx * armlen
    set h2cy cy + scale * dy * armlen
  ]
  [ set h2x1 x1-of hook2
    set h2y1 y1-of hook2
    set h2cx cx-of hook2
    set h2cy cy-of hook2
  ]
  set ratio1 armlen / armlen-of hook1
  set ratio2 armlen / armlen-of hook2
end

to lever!
  ifelse dragged = self
  [ ; if being dragged, then go there the mouse tells you
    set cx mouse-xcor
    set cy mouse-ycor
  ]
  [
    if not fixed?
    [ set xv ( xv * f ) + ( ( h1x2 - x1 ) + ( h2x1 - x2 ) ) * .5 * k
      set yv ( yv * f ) + ( ( h1y2 - y1 ) + ( h2y1 - y2 ) ) * .5 * k - g * armlen

      set cx cx + xv
      set cy cy + yv

      if abs cx > screen-edge-x [ set xv xv * -.99 set cx screen-edge-x * sign cx + xv bonk! (y2note cx)  ]
      if abs cy > screen-edge-y [ set yv yv * -.99 set cy screen-edge-y * sign cy + yv bonk! (x2note cy)  ]
    ]
  ]
  ifelse stiff?
  [ new-heading-2 ]
  [ new-heading-1 ]
  if spin? != 0
  [ let spin-rate spin? * spin-speed
    if spin*arm? [ set spin-rate spin-rate * armlen ]
    set heading heading + spin-rate
  ]
  setxy cx cy
end

to new-heading-1
  ; calculate new heading of lever
  ; if node2 of hook1 is overlapping center, keep same heading
  ; other wise, find heading to node2 of hook1
  ifelse ( distancexy-nowrap h1x2 h1y2 = 0 )
  [ set from-h1 180 + heading ]
  [ set from-h1 180 + towardsxy-nowrap h1x2 h1y2 ]
  ; likewise node1 of hook2
  ifelse ( distancexy-nowrap h2x1 h2y1 = 0 )
  [ set to-h2 heading ]
  [ set to-h2 towardsxy-nowrap h2x1 h2y1 ]
  ; calc heading
  set heading mean-heading from-h1 to-h2
end

to new-heading-2
  ; calculate new heading of lever
  ; if center of hook1 is overlapping center, keep same heading
  ; other wise, find heading to center of hook1
  ifelse ( distancexy-nowrap h1cx h1cy = 0 )
  [ set from-h1 180 + heading ]
  [ set from-h1 180 + towardsxy-nowrap h1cx h1cy ]
  ; likewise node1 of hook2
  ifelse ( distancexy-nowrap h2cx h2cy = 0 )
  [ set to-h2 heading ]
  [ set to-h2 towardsxy-nowrap h2cx h2cy ]
  ; calc heading
  set heading mean-heading from-h1 to-h2
end

to set-fixed==>on
  set fixed? true
  set -f "-f"
end

to set-fixed==>off
  set fixed? false
  set -f ""
end

to set-fixed==>toggle
  set fixed? not fixed?
  set -f ifelse-value fixed? [ "-f" ] [ "" ]
end

to set-spin==>on-cw ; clockwise
  set spin? 1
  set -r "-cw"
end

to set-spin==>on-ws ; widdershins (counter-clockwise)
  set spin? -1
  set -r "-ws"
end

to set-spin==>off
  set spin? 0
  set -r ""
end

to save-setup
  let filename user-choose-new-file
  if filename != false
  [ export-world filename ]
end

to load-setup
  let filename user-choose-file
  if filename != false
  [ import-world filename ]
end

to select-lever [ which-lever ]
  ; which-lever is -1 (previous) or +1 (next)
  ; no error checking on that, assume caller uses correctly
  ; illegal value will default to next
  ifelse not any? levers
  [ set selected nobody
    set old-selected nobody
  ]
  [ ifelse which-lever = -1
    [ select-lever-prev ]
    [ select-lever-next ]
  ]
end

;; NOTE: the following routines may seem needlessly complex,
;; but this is to allow for the future extension of
;; adding or removing levers at will while
;; the simulation is running.
;; So, instead of simply moving the selection
;; from lever who to lever who - 1 or who + 1,
;; we look for the lever with the next lower
;; or next higher who number.
;; if none (ie, this is the first or last lever),
;; we look for the lever with the highest or lowest
;; who number (not just lever 0 or lever (population - 1),
;; as lever 0 or lever (population - 1) may have been removed
;; or more levers added.

to select-lever-prev
  ; selects the previous lever
  ; use safety-wrapper "select-lever" to call this
  ; to make sure there are levers

  ; if nobody is currently selected, select the first lever
  ifelse not is-turtle? selected or selected = nobody
  [ ; no current selection, so...
      select-first-lever
  ]
  [ ; otherwise, remember prior selection
    set old-selected selected
    ; find next lowest numbered lever
    ; first find all lower-numbered levers
    let prev-levers levers with [who < who-of selected]
    ; are there any?
    ifelse any? prev-levers
    [ ; yes, so find the one with the largest who number
      ; (smaller than selected who number)
      set selected max-one-of prev-levers [who]
    ]
    [ ; no, so the selected lever must be the "first" lever, so...
      select-last-lever
    ]
  ]
end

to select-lever-next
  ; selects the next lever
  ; use safety-wrapper "select-lever" to call this
  ; to make sure there are levers

  ; if nobody is currently selected, select the last lever
  ifelse not is-turtle? selected or selected = nobody
  [ ; no current selection, so...
    select-last-lever
  ]
  [ ; otherwise, remember prior selection
    set old-selected selected
    ; find next highest numbered lever
    let next-levers levers with [ who > who-of selected ]
    ; are there any?
    ifelse any? next-levers
    [ ; yes, so, find the one with the smallest who number
      ; (larger than selected who number)
      set selected min-one-of next-levers [ who ]
    ]
    [ ; no, so the selected lever must be the "last" lever, so...
      select-first-lever
    ]
  ]
end

to-report first-lever
  report min-one-of levers [ who ]
end

to-report last-lever
  report max-one-of levers [ who ]
end

to select-first-lever
  set selected first-lever
  set old-selected nobody
end

to select-last-lever
  set selected last-lever
  set old-selected nobody
end

to bonk! [note-value]
  play-note item ((who + instrument) mod length instruments) instruments (note-value) velocity .05
end

to-report x2note [coord]
  report (16 + int ( 96 * (xcor + screen-edge-x ) / screen-size-x) )
end

to-report y2note [coord]
  report (16 + int ( 96 * (ycor + screen-edge-y ) / screen-size-y) )
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the projects:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS and INTEGRATED SIMULATION AND MODELING ENVIRONMENT.
; The project gratefully acknowledges the support of the
; National Science Foundation (REPP & ROLE programs) -- grant numbers 
; REC #9814682 and REC-0126227.
;
; Copyright 2005 by Uri Wilensky.
; Includes code by James P. Steiner.
;
; This work is licensed under the Creative Commons
; Attribution-NonCommercial-ShareAlike 2.5 License.  To view a copy of
; this license, visit http://creativecommons.org/licenses/by-nc-sa/2.5/
; or send a letter to Creative Commons, 559 Nathan Abbott Way,
; Stanford, California 94305, USA.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2005).  NetLogo Sound Machines model.
; http://ccl.northwestern.edu/netlogo/models/SoundMachines.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2005 Uri Wilensky.  Some rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/SoundMachines
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
290
10
700
441
12
12
16.0
1
11
1
1
1
0
0
0
1

CC-WINDOW
5
482
902
577
Command Center
0

BUTTON
11
66
125
99
NIL
setup-dangle
NIL
1
T
OBSERVER
T
NIL

SLIDER
721
50
893
83
f
f
0.9
0.9999
0.9783
1.0E-4
1
NIL

BUTTON
35
171
193
227
NIL
go
T
1
T
OBSERVER
T
NIL

SLIDER
11
29
226
62
num-levers
num-levers
4
100
12
1
1
NIL

SLIDER
721
118
893
151
scale
scale
0.1
12
1.7
0.1
1
NIL

SLIDER
721
84
893
117
g
g
0
0.18
0.031
1.0E-4
1
NIL

BUTTON
13
267
75
300
fix/free
ask selected [ set-fixed==>toggle ]
NIL
1
T
OBSERVER
T
NIL

BUTTON
76
301
138
334
arm +++
ask selected [ set armlen precision ( armlen + .1 ) 3 ]
NIL
1
T
OBSERVER
T
NIL

BUTTON
76
267
138
300
arm ---
ask selected [ if armlen > .1 [ set armlen precision ( armlen - .1 ) 3 ] ]
NIL
1
T
OBSERVER
T
NIL

BUTTON
13
301
75
334
arm reset
ask selected [ set armlen .5 ]
NIL
1
T
OBSERVER
T
NIL

BUTTON
200
301
255
334
next
select-lever-next
NIL
1
T
OBSERVER
T
NIL

SLIDER
20
363
243
396
spin-speed
spin-speed
-25
25
6.0
0.01
1
NIL

BUTTON
139
267
199
300
spin +1?
ask selected [ ifelse spin? != 1 [ set-spin==>on-cw ][set-spin==>off] ]
NIL
1
T
OBSERVER
NIL
NIL

MONITOR
206
175
279
224
NIL
ticks
0
1

BUTTON
126
66
279
99
NIL
setup-crazy-machine
NIL
1
T
OBSERVER
T
NIL

SWITCH
721
152
893
185
stiff?
stiff?
1
1
-1000

BUTTON
172
397
243
430
spd=0
set spin-speed 0
NIL
1
T
OBSERVER
T
NIL

TEXTBOX
11
249
147
267
modify selected lever:

BUTTON
76
397
131
430
+.5
set spin-speed\n      precision ((round (spin-speed * 2)) / 2.0 + 0.5)\n                1
NIL
1
T
OBSERVER
T
NIL

BUTTON
20
397
75
430
-.5
set spin-speed\n      precision ((round (spin-speed * 2)) / 2.0 - 0.5)\n                1
NIL
1
T
OBSERVER
T
NIL

BUTTON
139
301
199
334
spin -1?
ask selected [ ifelse spin? != -1 [ set-spin==>on-ws ][set-spin==>off] ]
NIL
1
T
OBSERVER
NIL
NIL

BUTTON
200
267
255
300
prev
set selected turtle ( ( (who-of selected) - 1 ) mod num-levers )
NIL
1
T
OBSERVER
T
NIL

BUTTON
11
100
164
133
NIL
setup-chaos-tentacle
NIL
1
T
OBSERVER
T
NIL

SWITCH
20
432
129
465
spin*arm?
spin*arm?
1
1
-1000

TEXTBOX
134
434
239
465
(multiplies spin rate * arm length)

BUTTON
717
405
804
438
NIL
save-setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
806
405
893
438
NIL
load-setup
NIL
1
T
OBSERVER
T
NIL

SLIDER
720
215
892
248
velocity
velocity
0
128
90.0
0.5
1
NIL

SLIDER
720
248
892
281
instrument
instrument
0
127
32
1
1
NIL

MONITOR
720
281
892
330
First Instrument
item instrument instruments
0
1

MONITOR
720
329
892
378
Last Instrument
item ((instrument + num-levers) mod length instruments) instruments
3
1

TEXTBOX
4
11
154
29
setup:

TEXTBOX
12
152
162
170
go:

TEXTBOX
719
32
869
50
behavior parameters:

TEXTBOX
717
197
867
215
sound parameters:

TEXTBOX
714
387
864
405
saving and loading:

TEXTBOX
302
450
638
468
You may use the mouse to select levers and move them.

TEXTBOX
14
344
164
362
spin controls:

@#$#@#$#@
WHAT IS IT?
-----------
This model shows one way turtles can make interesting and varied sounds, or if you like, music.  It uses some simple physics to make "machines" that twist, spin, turn, twitch, and bounce.  When a part of the machine touches a wall, ceiling, or floor, it makes a sound.  The pitch of the sound depends on the location of the touch.

You can start with a standard machine, or generate a random one.  You can change and build machines yourself, by changing the characteristics of each part.  You can sit back and let the machines play themselves, or you can use the mouse to move them around to control the sound yourself.


HOW IT WORKS
------------
The machines are made up of levers.  Levers can be different sizes and are pulled down by gravity (in proportion to their size) and subject to friction.

Each lever has one or two "hooks" to connect it to adjacent levers.  Each lever rather simple-mindedly tries to move and rotate to keep its hooks attached (or at least pointing towards) its neighbors' hooks.

Some levers have additional constraints.  A lever can be fixed in position so it can't move.  It can also rotate, as if powered by a motor.

The rules aren't perfect; for example, sometimes the levers separate.  Nonetheless, the behavior is often surprisingly realistic, and is rich enough to generate interesting motion which generates interesting sounds.


HOW TO USE IT
-------------
Press one of the SETUP... buttons to create a machine.  If you want, change NUM-LEVERS first to get more or fewer levers.

Press GO to start the machine going.  You should start hearing sounds, although if your machine never touches the wall (or ceiling or floor), you won't hear anything.

You can use the mouse to select a lever by clicking on it, or to move a lever by dragging it.  The selected lever has a circle around it.

Once you have selected a lever, a variety of controls on the left let you change that lever's properties.  They are not described in detail here; play with him and see for yourself what they do.

Some parameters affecting the entire system are on the bottom left and the right.  f is friction, g is gravity.  There are also controls for spin.  The STIFF? switch makes a big change in the rules the levers follow.

You can also use the VELOCITY slider to control the loudness of the sounds, and the INSTRUMENT slider to pick a different musical instrument for the levers to play.

If you make a machine you especially like, you can use the SAVE-SETUP button to keep it for later.  Use the LOAD-SETUP button to load it back in.


THINGS TO NOTICE
----------------
The simple rules followed by the levers produce some quite realistic-looking behavior.

Some machines don't do anything.  Other machines make simple, repetitive motions.  And others are wildly chaotic.


THINGS TO TRY
-------------
Try all of the different SETUP buttons.

Try all of the lever controls on the left.

Explore the effect of the parameters on the right.

See if you can make machines which are somewhat repetitive, but also somewhat unpredictable and chaotic.  Sometimes these make the most pleasing "music".


EXTENDING THE MODEL
-------------------
There are endless possibilities for how the same basic physics engine could be used to generate sound.  The levers could:
- Play different instruments, instead of all the same one.
- Make percussion sounds in addition pitched sounds.
- Make sounds even when they don't hit the wall
- Make sounds that depend on their characteristics
And so on.

How "normal" and "musical" can you make the model sound?  Currently the rules of the model aren't based on scales or tonality; by choosing pitches more carefully, you might be able to produce music which is less strange, more like music composed by humans.  You could also try to produce more regular rhythms by timing sounds to coincide with a regular beat.

Or, you could take the opposite approach, and see how wild, different, and unusual can you make the sounds.  Make something that doesn't sound like anything you've ever heard before!


NETLOGO FEATURES
----------------
The model uses NetLogo's sound extension.  The sound extension is described in the Sound section of the NetLogo User Manual.


RELATED MODELS
--------------
Beatbox
Composer
GasLab With Sound
Sound Workbench
Percussion Workbench


CREDITS AND REFERENCES
----------------------
This model is a streamlined variant of the Machines 2005 model created and submitted to the NetLogo User Community Models repository by James Steiner.  Machines 2005 is available from http://ccl.northwestern.edu/netlogo/models/community/machines-2005.

Thanks to James for creating an earlier version which was silent; Seth Tisue for first adding sound to it; and James again for his further improvements, and for releasing the model under a Creative Commons License.

This variant is copyright 2005 Uri Wilensky.  Includes code by James P. Steiner.  This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 License.  To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/2.5/ or send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.

To refer to this model in academic publications, please use: Wilensky, U. (2005). NetLogo Sound Machines model. http://ccl.northwestern.edu/netlogo/models/SoundMachines. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2005 Uri Wilensky. Some rights reserved. See http://ccl.northwestern.edu/netlogo/models/SoundMachines for terms of use.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

ball
true
10
Circle -13345367 true true 15 15 270

lever
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105

lever-cw
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Line -13345367 true 61 30 240 30
Line -13345367 true 240 30 195 44
Line -13345367 true 238 29 193 15
Line -13345367 true 239 269 61 270
Line -13345367 true 61 270 106 255
Line -13345367 true 104 285 61 271

lever-cw-s
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Line -13345367 true 61 30 240 30
Line -13345367 true 240 30 195 44
Line -13345367 true 238 29 193 15
Line -13345367 true 239 269 61 270
Line -13345367 true 61 270 106 255
Line -13345367 true 104 285 61 271
Circle -13345367 false true 29 29 242

lever-f
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Rectangle -13345367 true true 135 135 166 166

lever-f-cw
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Line -13345367 true 61 30 240 30
Line -13345367 true 240 30 195 44
Line -13345367 true 238 29 193 15
Line -13345367 true 239 269 61 270
Line -13345367 true 61 270 106 255
Line -13345367 true 104 285 61 271
Rectangle -13345367 true true 135 135 165 165

lever-f-cw-s
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Line -13345367 true 61 30 240 30
Line -13345367 true 240 30 195 44
Line -13345367 true 238 29 193 15
Line -13345367 true 239 269 61 270
Line -13345367 true 61 270 106 255
Line -13345367 true 104 285 61 271
Rectangle -13345367 true true 135 135 165 165
Circle -13345367 false true 29 30 240

lever-f-s
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Circle -13345367 false true 30 30 240
Rectangle -13345367 true true 135 135 165 165

lever-f-ws
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Line -13345367 true 120 30 181 30
Line -13345367 true 181 30 233 54
Line -13345367 true 120 30 68 52
Line -13345367 true 68 52 91 15
Line -13345367 true 69 53 111 60
Line -13345367 true 120 270 180 270
Line -13345367 true 179 271 233 246
Line -13345367 true 233 246 210 285
Line -13345367 true 233 245 186 241
Line -13345367 true 120 270 68 247
Rectangle -13345367 true true 135 135 166 166

lever-f-ws-s
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Line -13345367 true 120 30 181 30
Line -13345367 true 181 30 233 54
Line -13345367 true 120 30 68 52
Line -13345367 true 68 52 91 15
Line -13345367 true 69 53 111 60
Line -13345367 true 120 270 180 270
Line -13345367 true 179 271 233 246
Line -13345367 true 233 246 210 285
Line -13345367 true 233 245 186 241
Line -13345367 true 120 270 68 247
Circle -13345367 false true 30 30 240
Rectangle -13345367 true true 135 135 166 166

lever-s
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Circle -13345367 false true 30 30 240

lever-ws
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Line -13345367 true 120 30 181 30
Line -13345367 true 181 30 233 54
Line -13345367 true 120 30 68 52
Line -13345367 true 68 52 91 15
Line -13345367 true 69 53 111 60
Line -13345367 true 120 270 180 270
Line -13345367 true 179 271 233 246
Line -13345367 true 233 246 210 285
Line -13345367 true 233 245 186 241
Line -13345367 true 120 270 68 247

lever-ws-s
true
10
Circle -13345367 true true 120 -30 60
Circle -13345367 true true 121 270 58
Line -13345367 true 150 30 150 269
Polygon -13345367 true true 150 90 195 105 149 30 105 105
Line -13345367 true 120 30 181 30
Line -13345367 true 181 30 233 54
Line -13345367 true 120 30 68 52
Line -13345367 true 68 52 91 15
Line -13345367 true 69 53 111 60
Line -13345367 true 120 270 180 270
Line -13345367 true 179 271 233 246
Line -13345367 true 233 246 210 285
Line -13345367 true 233 245 186 241
Line -13345367 true 120 270 68 247
Circle -13345367 false true 30 30 240

@#$#@#$#@
NetLogo 3.0beta6
@#$#@#$#@
setup-crazy-machine
repeat 10 [ go ]
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
