breeds
  [
   electrons
   anodes
   cathodes
  ]

globals
  [
   charge-flow   ;; total number of electrons exiting the wire via the cathode
   ticks
   dt
  ]


;;;;;;;;;;;;;;;;;;;;;;;;
;;; Setup Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;

to setup
  ca
  set charge-flow 0
  set ticks 0
  set dt 1
  set-default-shape electrons "circle"
  set-default-shape cathodes "circle"
  set-default-shape anodes "circle"
  create-custom-electrons total-electrons
    [
      set color red
      setxy (random-float screen-edge-x - 10)
            (random-float screen-edge-y -  2)
    ]
  ask patches with [pxcor = screen-edge-x]
    [ sprout-anodes 1 [ ht ] ]
  ask patches with [pxcor = (- screen-edge-x)]
    [ sprout-cathodes 1 [ ht ] ]
  ask patches with [pxcor = (- screen-edge-x)]
    [ set pcolor blue ]
  ask patches with [pxcor = screen-edge-x]
    [ set pcolor red ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Runtime Procedures ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;

to go
  set ticks ticks + 1
  without-interruption
    [
      ask turtles with [pcolor = blue]
        [ ht ]
      ask electrons
        [ repel-anode ]
      ask electrons with [xcor > (- screen-edge-x)]
        [ attract ]
      ask electrons
        [ ask electrons with [distance myself > 0]
            [ repel ] ]
      ask electrons
        [ update-position ]
      do-plot
    ]
end

to do-plot
  set-current-plot "Current in the wire"
  plotxy ticks current
  set-current-plot "# of charges (cathode)"
  set-current-plot-pen "electrons"
  plotxy ticks charge-flow
end

;; electron procedure
to attract
  set heading 270
  safe-jump (0.5 * potential-diff / ( ( distancexy ( - screen-edge-x ) ycor ) ^ 2 ) )
end

;; electron procedure
to repel-anode
  set heading 270
  safe-jump ( 0.5 * potential-diff / ( ( distancexy screen-edge-x ycor ) ^ 2 ) )
end

;; electron procedure
to repel
  set heading towards myself
  rt 180
  safe-jump ( electron-mobility / ( ( 10 * distance-nowrap myself ) ^ 2 ) )
end

;; electron procedure
to safe-jump [ dist ]
  ;; if we wrap past the cathode, exit the wire
  let dest patch-at-heading-and-distance heading dist
  ifelse ( distance-nowrap dest != distance dest )
    [ exit-wire ]
    [ jump dist * dt]
end

;; electron procedure
to exit-wire
  ;; count ourselves as having left the wire, and then
  ;; jump over to the anode end of the wire
  set charge-flow charge-flow + 1
  hatch 1 [ set xcor screen-edge-x - 0.5
            set ycor random screen-edge-y * 2 ]
  die
end

to update-position
 ask cathodes
   [ ask electrons-here
       [ exit-wire ] ]
end

to-report current
  report charge-flow / ticks
end


; Copyright 2005 by Uri Wilensky.  Updated 2005.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2005).  NetLogo Conductor model.
; http://ccl.northwestern.edu/netlogo/models/Conductor.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2005 Uri Wilensky.  All rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/Conductor
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
238
9
698
130
22
4
10.0
1
30
1
1
1
0
0
1
1

CC-WINDOW
5
441
791
536
Command Center
0

BUTTON
31
74
115
114
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
117
74
202
114
NIL
go
T
1
T
OBSERVER
NIL
NIL

MONITOR
244
172
397
221
# of charges (cathode)
charge-flow
3
1

SLIDER
24
153
209
186
potential-diff
potential-diff
10
90
90
40
1
NIL

SLIDER
24
119
209
152
electron-mobility
electron-mobility
1
10
10
1
1
NIL

PLOT
231
227
502
427
# of charges (cathode)
time
charges
0.0
1.0
0.0
1.0
true
false
PENS
"default" 1.0 2 -2674135 false
"current" 1.0 0 -2674135 true
"electrons" 1.0 2 -16777216 true

MONITOR
509
172
644
221
current in the wire  
current
3
1

PLOT
505
228
782
425
Current in the wire
time
current
0.0
10.0
0.0
10.0
true
false
PENS
"default" 1.0 2 -2674135 true

SLIDER
24
36
209
69
total-electrons
total-electrons
150
200
200
10
1
NIL

TEXTBOX
239
131
307
149
Cathode\n

TEXTBOX
671
133
720
151
Anode\n

TEXTBOX
32
215
182
301
Note: current, electron mobility and potential difference are all expressed in arbitrary units in this model.

@#$#@#$#@
WHAT IS IT?
-----------
This model depicts a microscopic picture of current conduction within a wire connected across two ends (terminals) of a battery.  It shows how electric current and resistance in a wire emerges out of simple electrostatic interactions (i.e., interactions that obey Coulomb's Law, in this case) between the conduction electrons in the wire, and between the electrons and the two electrodes (i.e., terminals of the battery) across which the wire is connected.

This model allows you to see how the current flowing in a wire depends on:
a) the applied voltage across the battery terminals
b) the electron-mobility (i.e., the repulsion between electrons)
c) total number of free electrons in the wire
d) cross sectional area of the wire

This model is a part of the NIELS (NetLogo Investigations in Electromagnetism) curriculum.  The NIELS curriculum is currently under development at the CCL.


HOW IT WORKS
------------ 
For our purposes, assume we are using a copper wire.

Within the wire, there are "free" electrons -- i.e., each Cu atom contributes one outer conduction electron, which experience a negligible amount of attraction from the Cu nuclei.  All these "free" electrons can be assumed to behave like gas molecules, in absence of an electric field -- we will call it a "free electron gas".  In the absence of any electric (or magnetic) field, energies (and hence velocities) of these electrons are determined primarily by the temperature of the metal.

This model shows what happens when there is an electric field (E) due to the potential difference (V) between the battery terminals (anode and cathode) across which the wire is connected at a constant temperature.  The electric field (E) introduces a drift velocity in the electrons. In this model, electric field is generated by the attraction of the electrons towards the cathode and repulsion of electrons from the anode, both these forces obeying Coulomb's law.  The electrons also repel each other, obeying Coulomb'slLaw, and this repulsion between electrons gives rise to resistance.

This representation of resistance is an approximation -- in reality, resistance is a more complex phenomenon, depending also on scattering of electrons from atoms and crystal defects, and other complex interactions.

The total number of electrons remain constant during each run.


HOW TO USE IT
-------------
Select the total number of electrons by using the TOTAL-ELECTRONS slider.

When you press SETUP, electrons are created in the wire and the electrodes are created at the edges of the wire.

When you press GO, the model runs forever until you press GO once again when it stops.

The slider ELECTRON-MOBILITY changes the value of the Coulomb's force between the electrons.

The slider POTENTIAL-DIFF changes the potential difference between the cathode and the anode by changing the charge of each of them (both the cathode and the anode have equal but opposite charge), thereby, changing the Coulomb's force between the electrons and the electrodes.

The # OF CHARGES (CATHODE) plot shows the total number of electrons that have reached the CATHODE at a given instant of time.

Note: All units used in the model are arbitrary units; they are not related to real-world units.


THINGS TO NOTICE
----------------
The two plots show the number of electrons reaching the positive terminal of the battery as a function of time, and current as a function of time.  The later plot is the slope of the first one.

What is the general nature of the "Current in the wire" plot in all cases (i.e., does the plot resemble a circle or parabola or a straight line)?

What happens when you run the model for short periods of time (a few secs)?  What happens when you run the model for long periods of time (a few minutes)?

Does the model obey Ohm's law?


THINGS TO TRY
-------------
For fixed values of POTENTIAL-DIFF and TOTAL-ELECTRONS, run the model for different values of ELECTRON-MOBILITY.  Note down the value of current once it has stabilized (for each value of electron mobility) for each run.  Draw a plot (by hand) of Current vs. Electron Mobility.  What is the nature of the plot?  What does this plot tell you about the relationship between current and electron mobility?  How does this relationship compare with that between current and resistance in Ohm's law?

For a fixed value of ELECTRON-MOBILITY and TOTAL-ELECTRONS, run the model for different values of POTENTIAL-DIFF.  Note down the value of current once it has stabilized (for each value of electron mobility) for each run.  Draw a plot (by hand) of Current vs. Electron Mobility.  What is the nature of the plot?  What does this plot tell you about the relationship between current and POTENTIAL-DIFF?  How do you think current changes with POTENTIAL-DIFF when ELECTRON-MOBILITY is constant?

Keep everything else constant, just change the height of the wire (i.e., in a 3-D world, you are changing the cross section of the wire).  You can do this by using the downward pointing arrow in the control strip along the top edge of the view (if you place your mouse over the arrow it says "CHANGE THE HEIGHT OF THE WORLD").  How do you think cross sectional area of the wire affects current?

Keeping everything else constant, just change the TOTAL-ELECTRONS in the model and run it.  How does current depend on the total number of electrons in the wire?  How does resistance depend on it (no. of electrons in the wire)?


EXTENDING THE MODEL
------------------- 
Create some static nuclei and place them evenly within the wire. C reate a new rule for electrons now: write a sub-procedure in which the electrons collide with these nuclei, and the collisions are elastic, i.e., initial and final speeds of the electron are the same and the new heading of the electrons after collision is the difference of 180 and their old heading. 

Create a ON/OFF switch by which you can turn off the repulsion between electrons.

Afer doing the previous two suggestions, run the model now with:
a) both the rules (i.e., Collision and Inter-electronic Repulsion) "ON"
b) only one of the rules "ON"

Add a vertical velocity to the electrons. What happens to the current?


NETLOGO FEATURES
----------------
When a particle moves off of the left edge of the screen, it doesn't re-appear by wrapping onto the other side (as in most other NetLogo models).  However, electrons do wrap vertically.  This is handled using the SAFE-JUMP procedure.
 

RELATED MODELS
-------------- 
Electrostatics, Charged Particles


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2005). NetLogo Conductor model. http://ccl.northwestern.edu/netlogo/models/Conductor. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2005 Uri Wilensky. All rights reserved. See http://ccl.northwestern.edu/netlogo/models/Conductor for terms of use.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

cathode
false
0
Circle -7500403 false true 161 116 67

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0beta6
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
